@@ -1,4 +1,4 @@
-/* $Id: dot.c,v 1.60 2011/03/07 18:05:47 erg Exp $ $Revision: 1.60 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -22,16 +22,9 @@
 #include "gvc.h"
 #include "gvio.h"
 
-#ifndef WIN32_DLL
-#ifdef GVDLL
-__declspec(dllimport) boolean MemTest;
-#else
-#include "globals.h"
-#endif
-#endif
-
 #ifdef WIN32_DLL
 __declspec(dllimport) boolean MemTest;
+__declspec(dllimport) int GvExitOnUsage;
 /*gvc.lib cgraph.lib*/
 #ifdef WITH_CGRAPH
     #pragma comment( lib, "cgraph.lib" )
@@ -39,6 +32,8 @@ __declspec(dllimport) boolean MemTest;
     #pragma comment( lib, "graph.lib" )
 #endif
     #pragma comment( lib, "gvc.lib" )
+#else   /* not WIN32_DLL */
+#include "globals.h"
 #endif
 
 #include <stdlib.h>
@@ -130,6 +125,7 @@ static graph_t *create_test_graph(void)
 #define NUMNODES 5
 
     Agnode_t *node[NUMNODES];
+    Agedge_t *e;
     Agraph_t *g;
     Agraph_t *sg;
     int j, k;
@@ -152,7 +148,6 @@ static graph_t *create_test_graph(void)
 #else /* WITH_CGRAPH */
 	node[j] = agnode(g, name, 1);
 	agbindrec(node[j], "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
-
 #endif /* WITH_CGRAPH */
     }
 
@@ -162,13 +157,19 @@ static graph_t *create_test_graph(void)
 #ifndef WITH_CGRAPH
 	    agedge(g, node[j], node[k]);
 #else /* WITH_CGRAPH */
-	    agedge(g, node[j], node[k], NULL, 1);
+	    e = agedge(g, node[j], node[k], NULL, 1);
+	    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//edge custom data
 #endif /* WITH_CGRAPH */
 	}
     }
 
+#ifndef WITH_CGRAPH
     sg = agsubg (g, "cluster1");
     aginsert (sg, node[0]);
+#else /* WITH_CGRAPH */
+    sg = agsubg (g, "cluster1", 1);
+    agsubnode (sg, node[0], 1);
+#endif /* WITH_CGRAPH */
 
     return g;
 }
@@ -177,10 +178,9 @@ int main(int argc, char **argv)
 {
     graph_t *prev = NULL;
     int r, rc = 0;
-#ifndef WITH_CGRAPH
-#endif /* WITH_CGRAPH */
 
     Gvc = gvContextPlugins(lt_preloaded_symbols, DEMAND_LOADING);
+    GvExitOnUsage = 1;
     gvParseArgs(Gvc, argc, argv);
 #ifndef WIN32
     signal(SIGUSR1, gvToggle);
@@ -192,7 +192,7 @@ int main(int argc, char **argv)
 #endif
 
     if (MemTest) {
-	while (1) {
+	while (MemTest--) {
 	    /* Create a test graph */
 	    G = create_test_graph();
 
@@ -208,10 +208,6 @@ int main(int argc, char **argv)
     }
     else {
 	while ((G = gvNextInputGraph(Gvc))) {
-#ifdef WITH_CGRAPH
-
-
-#endif /* WITH_CGRAPH */
 	    if (prev) {
 		gvFreeLayout(Gvc, prev);
 		agclose(prev);
@@ -225,7 +221,6 @@ int main(int argc, char **argv)
     }
     gvFinalize(Gvc);
     
-
     r = gvFreeContext(Gvc);
     return (MAX(rc,r));
 }
