@@ -65,8 +65,6 @@ static char swork[maxlen];
     #pragma comment( lib, "gvortho.lib" )
     #pragma comment( lib, "sfdp.lib" )
     #pragma comment( lib, "rbtree.lib" )
-
-
 #endif   /* not WIN32_DLL */
 
 
@@ -182,14 +180,16 @@ static char* usestr =
     -b v - polygon line width, with v < 0 for no line. (0)\n\
     -c k - polygon color scheme (1)\n\
        0 : no polygons\n\
-       1 : pastel\n\
+       1 : pastel (default)\n\
        2 : blue to yellow\n\
        3 : white to red\n\
        4 : light grey to red\n\
        5 : primary colors\n\
        6 : sequential single hue red \n\
-       7 : sequential single hue lighter red \n\
-       8 : light grey\n\
+       7 : Adam color scheme\n\
+       8 : Adam blend\n\
+       9 : sequential single hue lighter red \n\
+      10 : light grey\n\
     -c_opacity=xx - 2-character hex string for opacity of polygons\n\
     -C k - generate at most k clusters. (0)\n\
     -d s - seed used to calculate Fielder vector for optimal coloring\n\
@@ -304,6 +304,7 @@ init(int argc, char **argv, params_t* pm)
   /*  bbox_margin[0] =  bbox_margin[1] = -0.2;*/
   pm->bbox_margin[0] =  pm->bbox_margin[1] = 0;
 
+  opterr = 0;
   while ((c = getopt(argc, argv, ":evODko:m:s:r:p:c:C:l:b:g:t:a:h:z:d:")) != -1) {
     switch (c) {
     case 'm':
@@ -428,6 +429,9 @@ init(int argc, char **argv, params_t* pm)
       if (pm->plot_label) free (pm->plot_label);
       pm->plot_label = strdup (optarg);
       break;
+    case ':':
+      fprintf(stderr, "gvpack: option -%c missing argument - ignored\n", optopt);
+      break;
     case '?':
       if (optopt == '?')
         usage(cmd, 0);
@@ -690,7 +694,7 @@ static void mapFromGraph (Agraph_t* g, params_t* pm)
   float* fsz;
 
   initDotIO(g);
-  graph = Import_coord_clusters_from_dot(g, pm->maxcluster, pm->dim, &n, &width, NULL, &x, &grouping, 
+  graph = Import_coord_clusters_from_dot(g, pm->maxcluster, pm->dim, &n, &width, &x, &grouping, 
 					   &rgb_r,  &rgb_g,  &rgb_b,  &fsz, &labels, pm->color_scheme, CLUSTERING_MODULARITY, pm->useClusters);
   makeMap (graph, n, x, width, grouping, labels, fsz, rgb_r, rgb_g, rgb_b, pm, g);
 }
