@@ -1,4 +1,4 @@
-/* $Id: circularinit.c,v 1.26 2011/02/15 16:27:53 erg Exp $ $Revision: 1.26 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -89,10 +89,9 @@ static node_t *makeDerivedNode(graph_t * dg, char *name, int isNode,
     ND_alg(n) = (void *) NEW(cdata);
     if (isNode) {
 	ND_pos(n) = N_NEW(Ndim, double);
-#if 0
-	ND_xsize(n) = ND_xsize((node_t *) orig);
-	ND_ysize(n) = ND_ysize((node_t *) orig);
-#endif
+	ND_lw(n) = ND_lw((node_t *) orig);
+	ND_rw(n) = ND_rw((node_t *) orig);
+	ND_ht(n) = ND_ht((node_t *) orig);
 	ORIGN(n) = (node_t *) orig;
     } else
 	ORIGG(n) = (graph_t *) orig;
@@ -123,6 +122,7 @@ Agraph_t **circomps(Agraph_t * g, int *cnt)
     dg = agopen("derived", AGFLAG_STRICT);
 #else /* WITH_CGRAPH */
     dg = agopen("derived", Agstrictundirected,NIL(Agdisc_t *));
+    agbindrec (dg, "info", sizeof(Agraphinfo_t), TRUE);
 #endif /* WITH_CGRAPH */
     GD_alg(g) = dg;  /* store derived graph for closing later */
 
@@ -233,7 +233,7 @@ void circoLayout(Agraph_t * g)
 	ccs = circomps(g, &ncc);
 
 	if (ncc == 1) {
-	    circularLayout(ccs[0]);
+	    circularLayout(ccs[0], g);
 	    copyPosns(ccs[0]);
 	    adjustNodes(g);
 	} else {
@@ -243,7 +243,7 @@ void circoLayout(Agraph_t * g)
 
 	    for (i = 0; i < ncc; i++) {
 		sg = ccs[i];
-		circularLayout(sg);
+		circularLayout(sg, g);
 		adjustNodes(sg);
 	    }
 	    /* FIX: splines have not been calculated for dg
@@ -271,6 +271,9 @@ void circo_layout(Agraph_t * g)
     dotneato_postprocess(g);
 }
 
+/* circo_cleanup:
+ * ND_alg is freed in circo_layout
+ */
 void circo_cleanup(graph_t * g)
 {
     node_t *n;
@@ -282,7 +285,7 @@ void circo_cleanup(graph_t * g)
 
     closeDerivedGraph((graph_t*)GD_alg(g));	/* delete derived graph */
 
-    free (ND_alg(n));
+    /* free (ND_alg(n)); */
     for (; n; n = agnxtnode(g, n)) {
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
 	    gv_cleanup_edge(e);
