@@ -1,18 +1,15 @@
-/* $Id: arrows.c,v 1.38 2009/10/05 13:06:08 ellson Exp $ $Revision: 1.38 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 #include "render.h"
@@ -27,7 +24,7 @@
 
 #define BITS_PER_ARROW 8
 
-#define BITS_PER_ARROW_TYPE 3
+#define BITS_PER_ARROW_TYPE 4
 /* arrow types (in BITS_PER_ARROW_TYPE bits) */
 #define ARR_TYPE_NONE	  (ARR_NONE)
 #define ARR_TYPE_NORM	  1
@@ -36,14 +33,16 @@
 #define ARR_TYPE_BOX	  4
 #define ARR_TYPE_DIAMOND  5
 #define ARR_TYPE_DOT      6
-/* Spare:  #define ARR_TYPE_xxx      7 */
+#define ARR_TYPE_CURVE    7
+#define ARR_TYPE_GAP      8
+/* Spare: 9-15 */
 
 /* arrow mods (in (BITS_PER_ARROW - BITS_PER_ARROW_TYPE) bits) */
 #define ARR_MOD_OPEN      (1<<(BITS_PER_ARROW_TYPE+0))
 #define ARR_MOD_INV       (1<<(BITS_PER_ARROW_TYPE+1))
 #define ARR_MOD_LEFT      (1<<(BITS_PER_ARROW_TYPE+2))
 #define ARR_MOD_RIGHT     (1<<(BITS_PER_ARROW_TYPE+3))
-/* Spare:  #define ARR_MOD_xxx       (1<<(BITS_PER_ARROW_TYPE+4))  */
+/* No spares */
 
 typedef struct arrowdir_t {
     char *dir;
@@ -88,7 +87,9 @@ static arrowname_t Arrownames[] = {
     {"box", ARR_TYPE_BOX},
     {"diamond", ARR_TYPE_DIAMOND},
     {"dot", ARR_TYPE_DOT},
-    {"none", ARR_TYPE_NONE},
+//    {"none", ARR_TYPE_NONE},
+    {"none", ARR_TYPE_GAP},
+//    {"gap", ARR_TYPE_GAP},
     /* ARR_MOD_INV is used only here to define two additional shapes
        since not all types can use it */
     {"inv", (ARR_TYPE_NORM | ARR_MOD_INV)},
@@ -101,6 +102,7 @@ static arrowname_t Arrownames[] = {
     /* Define "empty" as just "mpty" since "e" already taken as ARR_MOD_OPEN */
     /* Note that ARR_MOD_OPEN has expected meaning for ARR_TYPE_NORM shape */
     {"mpty", ARR_TYPE_NORM},
+    {"curve", ARR_TYPE_CURVE},
     {(char *) 0, ARR_TYPE_NONE}
 };
 
@@ -117,6 +119,8 @@ static void arrow_type_tee(GVJ_t * job, pointf p, pointf u, double arrowsize, do
 static void arrow_type_box(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag);
 static void arrow_type_diamond(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag);
 static void arrow_type_dot(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag);
+static void arrow_type_curve(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag);
+static void arrow_type_gap(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag);
 
 static arrowtype_t Arrowtypes[] = {
     {ARR_TYPE_NORM, 1.0, arrow_type_normal},
@@ -125,6 +129,8 @@ static arrowtype_t Arrowtypes[] = {
     {ARR_TYPE_BOX, 1.0, arrow_type_box},
     {ARR_TYPE_DIAMOND, 1.2, arrow_type_diamond},
     {ARR_TYPE_DOT, 0.8, arrow_type_dot},
+    {ARR_TYPE_CURVE, 1.0, arrow_type_curve},
+    {ARR_TYPE_GAP, 0.5, arrow_type_gap},
     {ARR_TYPE_NONE, 0.0, NULL}
 };
 
@@ -170,10 +176,13 @@ static void arrow_match_name(char *name, int *flag)
     int i, f;
 
     *flag = 0;
-    for (i = 0; *rest != '\0' && i < NUMB_OF_ARROW_HEADS; i++) {
+    for (i = 0; *rest != '\0' && i < NUMB_OF_ARROW_HEADS; ) {
 	f = ARR_TYPE_NONE;
         rest = arrow_match_shape(rest, &f);
-	*flag |= (f << (i * BITS_PER_ARROW));
+	if (f == ARR_TYPE_GAP && i == (NUMB_OF_ARROW_HEADS -1))
+	    f = ARR_TYPE_NONE;
+	if (f != ARR_TYPE_NONE)
+	    *flag |= (f << (i++ * BITS_PER_ARROW));
     }
 }
 
@@ -305,6 +314,107 @@ int arrowStartClip(edge_t* e, pointf * ps, int startp,
     return startp;
 }
 
+/* arrowOrthoClip:
+ * For orthogonal routing, we know each Bezier of spl is a horizontal or vertical
+ * line segment. We need to guarantee the B-spline stays this way. At present, we shrink
+ * the arrows if necessary to fit the last segment at either end. Alternatively, we could
+ * maintain the arrow size by dropping the 3 points of spl, and adding a new spl encoding
+ * the arrow, something "ex_0,y_0 x_1,y_1 x_1,y_1 x_1,y_1 x_1,y_1", when the last line
+ * segment is x_1,y_1 x_2,y_2 x_3,y_3 x_0,y_0. With a good deal more work, we could guarantee
+ * that the truncated spl clips to the arrow shape.
+ */
+void arrowOrthoClip(edge_t* e, pointf* ps, int startp, int endp, bezier* spl, int sflag, int eflag)
+{
+    pointf p, q, r, s, t;
+    double d, tlen, hlen, maxd;
+
+    if (sflag && eflag && (endp == startp)) { /* handle special case of two arrows on a single segment */
+	p = ps[endp];
+	q = ps[endp+3];
+	tlen = arrow_length (e, sflag);
+	hlen = arrow_length (e, eflag);
+        d = DIST(p, q);
+	if (hlen + tlen >= d) {
+	    hlen = tlen = d/3.0;
+	}
+	if (p.y == q.y) { /* horz segment */
+	    s.y = t.y = p.y;
+	    if (p.x < q.x) {
+		t.x = q.x - hlen;
+		s.x = p.x + tlen;
+	    }
+	    else {
+		t.x = q.x + hlen;
+		s.x = p.x - tlen;
+	    }
+	}
+	else {            /* vert segment */
+	    s.x = t.x = p.x;
+	    if (p.y < q.y) {
+		t.y = q.y - hlen;
+		s.y = p.y + tlen;
+	    }
+	    else {
+		t.y = q.y + hlen;
+		s.y = p.y - tlen;
+	    }
+	}
+	ps[endp] = ps[endp + 1] = s;
+	ps[endp + 2] = ps[endp + 3] = t;
+	spl->eflag = eflag, spl->ep = p;
+	spl->sflag = sflag, spl->sp = q;
+	return;
+    }
+    if (eflag) {
+	hlen = arrow_length(e, eflag);
+	p = ps[endp];
+	q = ps[endp+3];
+        d = DIST(p, q);
+	maxd = 0.9*d;
+	if (hlen >= maxd) {   /* arrow too long */
+	    hlen = maxd;
+	}
+	if (p.y == q.y) { /* horz segment */
+	    r.y = p.y;
+	    if (p.x < q.x) r.x = q.x - hlen;
+	    else r.x = q.x + hlen;
+	}
+	else {            /* vert segment */
+	    r.x = p.x;
+	    if (p.y < q.y) r.y = q.y - hlen;
+	    else r.y = q.y + hlen;
+	}
+	ps[endp + 1] = p;
+	ps[endp + 2] = ps[endp + 3] = r;
+	spl->eflag = eflag;
+	spl->ep = q;
+    }
+    if (sflag) {
+	tlen = arrow_length(e, sflag);
+	p = ps[startp];
+	q = ps[startp+3];
+        d = DIST(p, q);
+	maxd = 0.9*d;
+	if (tlen >= maxd) {   /* arrow too long */
+	    tlen = maxd;
+	}
+	if (p.y == q.y) { /* horz segment */
+	    r.y = p.y;
+	    if (p.x < q.x) r.x = p.x + tlen;
+	    else r.x = p.x - tlen;
+	}
+	else {            /* vert segment */
+	    r.x = p.x;
+	    if (p.y < q.y) r.y = p.y + tlen;
+	    else r.y = p.y - tlen;
+	}
+	ps[startp] = ps[startp + 1] = r;
+	ps[startp + 2] = q;
+	spl->sflag = sflag;
+	spl->sp = p;
+    }
+}
+
 static void arrow_type_normal(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag)
 {
     pointf q, v, a[5];
@@ -401,6 +511,17 @@ static void arrow_type_crow(GVJ_t * job, pointf p, pointf u, double arrowsize, d
 	gvrender_polygon(job, a, 9, 1);
 }
 
+static void arrow_type_gap(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag)
+{
+    pointf q, a[2];
+
+    q.x = p.x + u.x;
+    q.y = p.y + u.y;
+    a[0] = p;
+    a[1] = q;
+    gvrender_polyline(job, a, 2);
+}
+
 static void arrow_type_tee(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag)
 {
     pointf m, n, q, v, a[4];
@@ -502,6 +623,46 @@ static void arrow_type_dot(GVJ_t * job, pointf p, pointf u, double arrowsize, do
     gvrender_ellipse(job, AF, 2, !(flag & ARR_MOD_OPEN));
 }
 
+
+/* Draw a concave semicircle using a single cubic bezier curve that touches p at its midpoint.
+ * See http://digerati-illuminatus.blogspot.com.au/2008/05/approximating-semicircle-with-cubic.html for details.
+ */
+static void arrow_type_curve(GVJ_t* job, pointf p, pointf u, double arrowsize, double penwidth, int flag)
+{
+    double arrowwidth = penwidth > 4 ? 0.5 * penwidth / 4 : 0.5;
+    pointf q, v, w;
+    pointf AF[4], a[2];
+
+    q.x = p.x + u.x;
+    q.y = p.y + u.y; 
+    v.x = -u.y * arrowwidth; 
+    v.y = u.x * arrowwidth;
+    w.x = v.y; // same direction as u, same magnitude as v.
+    w.y = -v.x;
+    a[0] = p;
+    a[1] = q;
+
+    AF[0].x = p.x + v.x + w.x;
+    AF[0].y = p.y + v.y + w.y;
+
+    AF[3].x = p.x - v.x + w.x;
+    AF[3].y = p.y - v.y + w.y;
+
+    AF[1].x = p.x + 0.95 * v.x + w.x - w.x * 4.0 / 3.0;
+    AF[1].y = AF[0].y - w.y * 4.0 / 3.0;
+
+    AF[2].x = p.x - 0.95 * v.x + w.x - w.x * 4.0 / 3.0;
+    AF[2].y = AF[3].y - w.y * 4.0 / 3.0;
+
+    gvrender_polyline(job, a, 2);
+    if (flag & ARR_MOD_LEFT)
+	Bezier(AF, 3, 0.5, NULL, AF);
+    else if (flag & ARR_MOD_RIGHT)
+	Bezier(AF, 3, 0.5, AF, NULL);
+    gvrender_beziercurve(job, AF, sizeof(AF) / sizeof(pointf), FALSE, FALSE, FALSE);
+}
+
+
 static pointf arrow_gen_type(GVJ_t * job, pointf p, pointf u, double arrowsize, double penwidth, int flag)
 {
     int f;
