@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 2.5.  */
+/* A Bison parser, made by GNU Bison 2.6.1.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.5"
+#define YYBISON_VERSION "2.6.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -58,14 +58,11 @@
 /* Pull parsers.  */
 #define YYPULL 1
 
-/* Using locations.  */
-#define YYLSP_NEEDED 0
 
 
 
 /* Copy the first part of user declarations.  */
-
-/* Line 268 of yacc.c  */
+/* Line 336 of yacc.c  */
 #line 14 "../../lib/common/htmlparse.y"
 
 
@@ -470,14 +467,16 @@ popFont (void)
 }
 
 
+/* Line 336 of yacc.c  */
+#line 472 "y.tab.c"
 
-/* Line 268 of yacc.c  */
-#line 476 "y.tab.c"
-
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
+# ifndef YY_NULL
+#  if defined __cplusplus && 201103L <= __cplusplus
+#   define YY_NULL nullptr
+#  else
+#   define YY_NULL 0
+#  endif
+# endif
 
 /* Enabling verbose error messages.  */
 #ifdef YYERROR_VERBOSE
@@ -487,11 +486,17 @@ popFont (void)
 # define YYERROR_VERBOSE 0
 #endif
 
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 0
+/* In a future release of Bison, this section will be replaced
+   by #include "y.tab.h".  */
+#ifndef YY_Y_TAB_H
+# define YY_Y_TAB_H
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int htmldebug;
 #endif
-
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -573,12 +578,10 @@ popFont (void)
 
 
 
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 293 of yacc.c  */
+/* Line 350 of yacc.c  */
 #line 418 "../../lib/common/htmlparse.y"
 
   int    i;
@@ -590,21 +593,36 @@ typedef union YYSTYPE
   pitem*       p;
 
 
-
-/* Line 293 of yacc.c  */
-#line 596 "y.tab.c"
+/* Line 350 of yacc.c  */
+#line 598 "y.tab.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define htmlstype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+extern YYSTYPE htmllval;
+
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int htmlparse (void *YYPARSE_PARAM);
+#else
+int htmlparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int htmlparse (void);
+#else
+int htmlparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
+
+#endif /* !YY_Y_TAB_H  */
 
 /* Copy the second part of user declarations.  */
 
-
-/* Line 343 of yacc.c  */
-#line 608 "y.tab.c"
+/* Line 353 of yacc.c  */
+#line 626 "y.tab.c"
 
 #ifdef short
 # undef short
@@ -710,6 +728,7 @@ YYID (htmli)
 #    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 #     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
+      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
 #     ifndef EXIT_SUCCESS
 #      define EXIT_SUCCESS 0
 #     endif
@@ -801,20 +820,20 @@ union htmlalloc
 #endif
 
 #if defined YYCOPY_NEEDED && YYCOPY_NEEDED
-/* Copy COUNT objects from FROM to TO.  The source and destination do
+/* Copy COUNT objects from SRC to DST.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
 #  if defined __GNUC__ && 1 < __GNUC__
-#   define YYCOPY(To, From, Count) \
-      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
+#   define YYCOPY(Dst, Src, Count) \
+      __builtin_memcpy (Dst, Src, (Count) * sizeof (*(Src)))
 #  else
-#   define YYCOPY(To, From, Count)		\
-      do					\
-	{					\
-	  YYSIZE_T htmli;				\
-	  for (htmli = 0; htmli < (Count); htmli++)	\
-	    (To)[htmli] = (From)[htmli];		\
-	}					\
+#   define YYCOPY(Dst, Src, Count)              \
+      do                                        \
+        {                                       \
+          YYSIZE_T htmli;                         \
+          for (htmli = 0; htmli < (Count); htmli++)   \
+            (Dst)[htmli] = (Src)[htmli];            \
+        }                                       \
       while (YYID (0))
 #  endif
 # endif
@@ -926,7 +945,7 @@ static const htmltype_uint16 htmlrline[] =
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
+#if YYDEBUG || YYERROR_VERBOSE || 0
 /* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const htmltname[] =
@@ -941,7 +960,7 @@ static const char *const htmltname[] =
   "n_font", "italic", "n_italic", "bold", "n_bold", "underline",
   "n_underline", "sup", "n_sup", "sub", "n_sub", "br", "string", "table",
   "@1", "fonttable", "opt_space", "rows", "row", "$@2", "cells", "cell",
-  "$@3", "$@4", "$@5", "$@6", "image", "HR", "VR", 0
+  "$@3", "$@4", "$@5", "$@6", "image", "HR", "VR", YY_NULL
 };
 #endif
 
@@ -1137,17 +1156,18 @@ static const htmltype_uint8 htmlstos[] =
 
 #define YYRECOVERING()  (!!htmlerrstatus)
 
-#define YYBACKUP(Token, Value)					\
-do								\
-  if (htmlchar == YYEMPTY && htmllen == 1)				\
-    {								\
-      htmlchar = (Token);						\
-      htmllval = (Value);						\
-      YYPOPSTACK (1);						\
-      goto htmlbackup;						\
-    }								\
-  else								\
-    {								\
+#define YYBACKUP(Token, Value)                                  \
+do                                                              \
+  if (htmlchar == YYEMPTY)                                        \
+    {                                                           \
+      htmlchar = (Token);                                         \
+      htmllval = (Value);                                         \
+      YYPOPSTACK (htmllen);                                       \
+      htmlstate = *htmlssp;                                         \
+      goto htmlbackup;                                            \
+    }                                                           \
+  else                                                          \
+    {                                                           \
       htmlerror (YY_("syntax error: cannot back up")); \
       YYERROR;							\
     }								\
@@ -1157,32 +1177,33 @@ while (YYID (0))
 #define YYTERROR	1
 #define YYERRCODE	256
 
-
 /* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
    If N is 0, then set CURRENT to the empty location which ends
    the previous symbol: RHS[0] (always defined).  */
 
-#define YYRHSLOC(Rhs, K) ((Rhs)[K])
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)				\
-    do									\
-      if (YYID (N))                                                    \
-	{								\
-	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
-	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
-	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
-	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
-	}								\
-      else								\
-	{								\
-	  (Current).first_line   = (Current).last_line   =		\
-	    YYRHSLOC (Rhs, 0).last_line;				\
-	  (Current).first_column = (Current).last_column =		\
-	    YYRHSLOC (Rhs, 0).last_column;				\
-	}								\
+# define YYLLOC_DEFAULT(Current, Rhs, N)                                \
+    do                                                                  \
+      if (YYID (N))                                                     \
+        {                                                               \
+          (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;        \
+          (Current).first_column = YYRHSLOC (Rhs, 1).first_column;      \
+          (Current).last_line    = YYRHSLOC (Rhs, N).last_line;         \
+          (Current).last_column  = YYRHSLOC (Rhs, N).last_column;       \
+        }                                                               \
+      else                                                              \
+        {                                                               \
+          (Current).first_line   = (Current).last_line   =              \
+            YYRHSLOC (Rhs, 0).last_line;                                \
+          (Current).first_column = (Current).last_column =              \
+            YYRHSLOC (Rhs, 0).last_column;                              \
+        }                                                               \
     while (YYID (0))
 #endif
 
+#define YYRHSLOC(Rhs, K) ((Rhs)[K])
+
+
 
 /* This macro is provided for backward compatibility. */
 
@@ -1242,6 +1263,8 @@ html_symbol_value_print (htmloutput, htmltype, htmlvaluep)
     YYSTYPE const * const htmlvaluep;
 #endif
 {
+  FILE *htmlo = htmloutput;
+  YYUSE (htmlo);
   if (!htmlvaluep)
     return;
 # ifdef YYPRINT
@@ -1493,12 +1516,12 @@ static int
 htmlsyntax_error (YYSIZE_T *htmlmsg_alloc, char **htmlmsg,
                 htmltype_int16 *htmlssp, int htmltoken)
 {
-  YYSIZE_T htmlsize0 = htmltnamerr (0, htmltname[htmltoken]);
+  YYSIZE_T htmlsize0 = htmltnamerr (YY_NULL, htmltname[htmltoken]);
   YYSIZE_T htmlsize = htmlsize0;
   YYSIZE_T htmlsize1;
   enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
   /* Internationalized format string. */
-  const char *htmlformat = 0;
+  const char *htmlformat = YY_NULL;
   /* Arguments of htmlformat. */
   char const *htmlarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
   /* Number of reported tokens (one for the "unexpected", one per
@@ -1558,7 +1581,7 @@ htmlsyntax_error (YYSIZE_T *htmlmsg_alloc, char **htmlmsg,
                     break;
                   }
                 htmlarg[htmlcount++] = htmltname[htmlx];
-                htmlsize1 = htmlsize + htmltnamerr (0, htmltname[htmlx]);
+                htmlsize1 = htmlsize + htmltnamerr (YY_NULL, htmltname[htmlx]);
                 if (! (htmlsize <= htmlsize1
                        && htmlsize1 <= YYSTACK_ALLOC_MAXIMUM))
                   return 2;
@@ -1650,20 +1673,6 @@ htmldestruct (htmlmsg, htmltype, htmlvaluep)
 }
 
 
-/* Prevent warnings from -Wmissing-prototypes.  */
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int htmlparse (void *YYPARSE_PARAM);
-#else
-int htmlparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int htmlparse (void);
-#else
-int htmlparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
 
 /* The lookahead symbol.  */
@@ -1710,7 +1719,7 @@ htmlparse ()
        `htmlss': related to states.
        `htmlvs': related to semantic values.
 
-       Refer to the stacks thru separate pointers, to allow htmloverflow
+       Refer to the stacks through separate pointers, to allow htmloverflow
        to reallocate them elsewhere.  */
 
     /* The state stack.  */
@@ -1764,7 +1773,6 @@ htmlparse ()
      The wasted elements are never initialized.  */
   htmlssp = htmlss;
   htmlvsp = htmlvs;
-
   goto htmlsetstate;
 
 /*------------------------------------------------------------.
@@ -1942,148 +1950,127 @@ htmlreduce:
   switch (htmln)
     {
         case 2:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 450 "../../lib/common/htmlparse.y"
     { HTMLstate.lbl = mkLabel((htmlvsp[(2) - (3)].txt),HTML_TEXT); }
     break;
 
   case 3:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 451 "../../lib/common/htmlparse.y"
     { HTMLstate.lbl = mkLabel((htmlvsp[(2) - (3)].tbl),HTML_TBL); }
     break;
 
   case 4:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 452 "../../lib/common/htmlparse.y"
     { cleanup(); YYABORT; }
     break;
 
   case 5:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 455 "../../lib/common/htmlparse.y"
     { (htmlval.txt) = mkText(); }
     break;
 
   case 8:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 462 "../../lib/common/htmlparse.y"
     { appendFItemList(HTMLstate.str);}
     break;
 
   case 9:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 463 "../../lib/common/htmlparse.y"
     {appendFLineList((htmlvsp[(1) - (1)].i));}
     break;
 
   case 16:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 472 "../../lib/common/htmlparse.y"
     { pushFont ((htmlvsp[(1) - (1)].font)); }
     break;
 
   case 17:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 475 "../../lib/common/htmlparse.y"
     { popFont (); }
     break;
 
   case 18:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 478 "../../lib/common/htmlparse.y"
     {pushFont((htmlvsp[(1) - (1)].font));}
     break;
 
   case 19:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 481 "../../lib/common/htmlparse.y"
     {popFont();}
     break;
 
   case 20:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 484 "../../lib/common/htmlparse.y"
     {pushFont((htmlvsp[(1) - (1)].font));}
     break;
 
   case 21:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 487 "../../lib/common/htmlparse.y"
     {popFont();}
     break;
 
   case 22:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 490 "../../lib/common/htmlparse.y"
     {pushFont((htmlvsp[(1) - (1)].font));}
     break;
 
   case 23:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 493 "../../lib/common/htmlparse.y"
     {popFont();}
     break;
 
   case 24:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 496 "../../lib/common/htmlparse.y"
     {pushFont((htmlvsp[(1) - (1)].font));}
     break;
 
   case 25:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 499 "../../lib/common/htmlparse.y"
     {popFont();}
     break;
 
   case 26:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 502 "../../lib/common/htmlparse.y"
     {pushFont((htmlvsp[(1) - (1)].font));}
     break;
 
   case 27:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 505 "../../lib/common/htmlparse.y"
     {popFont();}
     break;
 
   case 28:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 508 "../../lib/common/htmlparse.y"
     { (htmlval.i) = (htmlvsp[(1) - (2)].i); }
     break;
 
   case 29:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 509 "../../lib/common/htmlparse.y"
     { (htmlval.i) = (htmlvsp[(1) - (1)].i); }
     break;
 
   case 32:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 516 "../../lib/common/htmlparse.y"
     { 
           if (nonSpace(agxbuse(HTMLstate.str))) {
@@ -2099,8 +2086,7 @@ htmlreduce:
     break;
 
   case 33:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 527 "../../lib/common/htmlparse.y"
     {
           if (nonSpace(agxbuse(HTMLstate.str))) {
@@ -2113,170 +2099,146 @@ htmlreduce:
     break;
 
   case 34:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 537 "../../lib/common/htmlparse.y"
     { (htmlval.tbl) = (htmlvsp[(1) - (1)].tbl); }
     break;
 
   case 35:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 538 "../../lib/common/htmlparse.y"
     { (htmlval.tbl)=(htmlvsp[(2) - (3)].tbl); }
     break;
 
   case 36:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 539 "../../lib/common/htmlparse.y"
     { (htmlval.tbl)=(htmlvsp[(2) - (3)].tbl); }
     break;
 
   case 37:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 540 "../../lib/common/htmlparse.y"
     { (htmlval.tbl)=(htmlvsp[(2) - (3)].tbl); }
     break;
 
   case 38:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 541 "../../lib/common/htmlparse.y"
     { (htmlval.tbl)=(htmlvsp[(2) - (3)].tbl); }
     break;
 
   case 41:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 548 "../../lib/common/htmlparse.y"
     { (htmlval.p) = (htmlvsp[(1) - (1)].p); }
     break;
 
   case 42:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 549 "../../lib/common/htmlparse.y"
     { (htmlval.p) = (htmlvsp[(2) - (2)].p); }
     break;
 
   case 43:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 550 "../../lib/common/htmlparse.y"
     { (htmlvsp[(1) - (3)].p)->ruled = 1; (htmlval.p) = (htmlvsp[(3) - (3)].p); }
     break;
 
   case 44:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 553 "../../lib/common/htmlparse.y"
     { addRow (); }
     break;
 
   case 45:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 553 "../../lib/common/htmlparse.y"
     { (htmlval.p) = lastRow(); }
     break;
 
   case 46:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 556 "../../lib/common/htmlparse.y"
     { (htmlval.cell) = (htmlvsp[(1) - (1)].cell); }
     break;
 
   case 47:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 557 "../../lib/common/htmlparse.y"
     { (htmlval.cell) = (htmlvsp[(2) - (2)].cell); }
     break;
 
   case 48:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 558 "../../lib/common/htmlparse.y"
     { (htmlvsp[(1) - (3)].cell)->ruled |= HTML_VRULE; (htmlval.cell) = (htmlvsp[(3) - (3)].cell); }
     break;
 
   case 49:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 561 "../../lib/common/htmlparse.y"
     { setCell((htmlvsp[(1) - (2)].cell),(htmlvsp[(2) - (2)].tbl),HTML_TBL); }
     break;
 
   case 50:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 561 "../../lib/common/htmlparse.y"
     { (htmlval.cell) = (htmlvsp[(1) - (4)].cell); }
     break;
 
   case 51:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 562 "../../lib/common/htmlparse.y"
     { setCell((htmlvsp[(1) - (2)].cell),(htmlvsp[(2) - (2)].txt),HTML_TEXT); }
     break;
 
   case 52:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 562 "../../lib/common/htmlparse.y"
     { (htmlval.cell) = (htmlvsp[(1) - (4)].cell); }
     break;
 
   case 53:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 563 "../../lib/common/htmlparse.y"
     { setCell((htmlvsp[(1) - (2)].cell),(htmlvsp[(2) - (2)].img),HTML_IMAGE); }
     break;
 
   case 54:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 563 "../../lib/common/htmlparse.y"
     { (htmlval.cell) = (htmlvsp[(1) - (4)].cell); }
     break;
 
   case 55:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 564 "../../lib/common/htmlparse.y"
     { setCell((htmlvsp[(1) - (1)].cell),mkText(),HTML_TEXT); }
     break;
 
   case 56:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 564 "../../lib/common/htmlparse.y"
     { (htmlval.cell) = (htmlvsp[(1) - (3)].cell); }
     break;
 
   case 57:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 567 "../../lib/common/htmlparse.y"
     { (htmlval.img) = (htmlvsp[(1) - (2)].img); }
     break;
 
   case 58:
-
-/* Line 1806 of yacc.c  */
+/* Line 1787 of yacc.c  */
 #line 568 "../../lib/common/htmlparse.y"
     { (htmlval.img) = (htmlvsp[(1) - (1)].img); }
     break;
 
 
-
-/* Line 1806 of yacc.c  */
-#line 2280 "y.tab.c"
+/* Line 1787 of yacc.c  */
+#line 2242 "y.tab.c"
       default: break;
     }
   /* User semantic actions sometimes alter htmlchar, and that requires
@@ -2463,7 +2425,7 @@ htmlabortlab:
   htmlresult = 1;
   goto htmlreturn;
 
-#if !defined(htmloverflow) || YYERROR_VERBOSE
+#if !defined htmloverflow || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | htmlexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2505,8 +2467,7 @@ htmlreturn:
 }
 
 
-
-/* Line 2067 of yacc.c  */
+/* Line 2048 of yacc.c  */
 #line 580 "../../lib/common/htmlparse.y"
 
 
