@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.6.1.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +30,15 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_Y_TAB_H
+# define YY_Y_TAB_H
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int htmldebug;
+#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -53,16 +60,26 @@
      T_n_italic = 269,
      T_n_bold = 270,
      T_n_underline = 271,
-     T_BR = 272,
-     T_br = 273,
-     T_IMG = 274,
-     T_img = 275,
-     T_table = 276,
-     T_cell = 277,
-     T_font = 278,
-     T_italic = 279,
-     T_bold = 280,
-     T_underline = 281
+     T_n_sup = 272,
+     T_n_sub = 273,
+     T_HR = 274,
+     T_hr = 275,
+     T_end_hr = 276,
+     T_VR = 277,
+     T_vr = 278,
+     T_end_vr = 279,
+     T_BR = 280,
+     T_br = 281,
+     T_IMG = 282,
+     T_img = 283,
+     T_table = 284,
+     T_cell = 285,
+     T_font = 286,
+     T_italic = 287,
+     T_bold = 288,
+     T_underline = 289,
+     T_sup = 290,
+     T_sub = 291
    };
 #endif
 /* Tokens.  */
@@ -80,26 +97,34 @@
 #define T_n_italic 269
 #define T_n_bold 270
 #define T_n_underline 271
-#define T_BR 272
-#define T_br 273
-#define T_IMG 274
-#define T_img 275
-#define T_table 276
-#define T_cell 277
-#define T_font 278
-#define T_italic 279
-#define T_bold 280
-#define T_underline 281
-
+#define T_n_sup 272
+#define T_n_sub 273
+#define T_HR 274
+#define T_hr 275
+#define T_end_hr 276
+#define T_VR 277
+#define T_vr 278
+#define T_end_vr 279
+#define T_BR 280
+#define T_br 281
+#define T_IMG 282
+#define T_img 283
+#define T_table 284
+#define T_cell 285
+#define T_font 286
+#define T_italic 287
+#define T_bold 288
+#define T_underline 289
+#define T_sup 290
+#define T_sub 291
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 1676 of yacc.c  */
-#line 409 "../../lib/common/htmlparse.y"
+/* Line 2049 of yacc.c  */
+#line 418 "../../lib/common/htmlparse.y"
 
   int    i;
   htmltxt_t*  txt;
@@ -107,11 +132,11 @@ typedef union YYSTYPE
   htmltbl_t*   tbl;
   htmlfont_t*  font;
   htmlimg_t*   img;
+  pitem*       p;
 
 
-
-/* Line 1676 of yacc.c  */
-#line 115 "y.tab.h"
+/* Line 2049 of yacc.c  */
+#line 140 "y.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define htmlstype YYSTYPE /* obsolescent; will be withdrawn */
@@ -120,4 +145,18 @@ typedef union YYSTYPE
 
 extern YYSTYPE htmllval;
 
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int htmlparse (void *YYPARSE_PARAM);
+#else
+int htmlparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int htmlparse (void);
+#else
+int htmlparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
+#endif /* !YY_Y_TAB_H  */
