@@ -1,18 +1,15 @@
-/* $Id: htmltable.h,v 1.32 2009/07/14 13:18:07 ellson Exp $ $Revision: 1.32 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #ifdef __cplusplus
 extern "C" {
@@ -38,41 +35,14 @@ extern "C" {
 
 #define UNSET_ALIGN 0
 
-/* Bold, Italic, Underline */
-#define HTML_BF 1
-#define HTML_IF 2
-#define HTML_UL 4
-
-    /* font information
-     * If name or color is NULL, or size < 0, that attribute
-     * is unspecified. 
-     */
-    typedef struct {
-	char*  name;
-	char*  color;
-        int    flags:7;  /* HTML_UL, HTML_IF, HTML_BF */
-	int    cnt;   /* reference count */
-	double size;
-    } htmlfont_t;
-
     /* paras of text within a cell
      * NOTE: As required, the str field in para is utf-8.
      * This translation is done when libexpat scans the input.
      */
 	
-    /* atomic unit of text emitted using a single htmlfont_t */
+    /* line of textpara_t's */
     typedef struct {
-	char *str;
-	PostscriptAlias *postscript_alias;
-	void *layout;
-	void (*free_layout) (void *layout);
-	htmlfont_t *font;
-	double size, yoffset_layout, yoffset_centerline;
-    } textitem_t;
-	
-    /* line of textitems_t */
-    typedef struct {
-	textitem_t *items;
+	textpara_t *items;
 	short nitems;
 	char just;
 	double size;   /* width of para */
@@ -82,6 +52,7 @@ extern "C" {
     typedef struct {
 	htextpara_t *paras;
 	short nparas;
+	char simple;
 	boxf box;
     } htmltxt_t;
 
@@ -96,8 +67,10 @@ extern "C" {
 	char *port;
 	char *target;
 	char *title;
+	char *id;
 	char *bgcolor;
 	char *pencolor;
+	int gradientangle;
 	signed char space;
 	unsigned char border;
 	unsigned char pad;
@@ -105,6 +78,7 @@ extern "C" {
 	unsigned short flags;
 	unsigned short width;
 	unsigned short height;
+	unsigned short style;
 	boxf box;		/* its geometric placement in points */
     } htmldata_t;
 
@@ -113,6 +87,9 @@ extern "C" {
 #define HTML_TEXT 2
 #define HTML_IMAGE 3
 
+#define HTML_VRULE 1
+#define HTML_HRULE 2
+
     typedef struct htmlcell_t htmlcell_t;
     typedef struct htmltbl_t htmltbl_t;
 	
@@ -134,6 +111,7 @@ extern "C" {
 	int rc;			/* number of rows */
 	int cc;			/* number of columns */
 	htmlfont_t *font;	/* font info */
+	unsigned char flags;
     };
 
     struct htmllabel_t {
@@ -153,6 +131,7 @@ extern "C" {
 	unsigned short row;
 	htmllabel_t child;
 	htmltbl_t *parent;
+	unsigned char ruled;
     };
 
 /* During parsing, table contents are stored as rows of cells.
@@ -166,6 +145,7 @@ extern "C" {
 	    Dt_t *rp;
 	    htmlcell_t *cp;
 	} u;
+	unsigned char ruled;
     } pitem;
 	
     extern htmllabel_t *parseHTML(char *, int *, int);
