@@ -1,18 +1,15 @@
-/* $Id: input.c,v 1.112 2009/12/01 16:52:08 ellson Exp $ $Revision: 1.112 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #include <ctype.h>
 #include "render.h"
@@ -56,17 +53,22 @@ static char *fdpItems = "\n\
  -LC<v>      - Set overlap expansion factor to v\n\
  -LT[*]<v>   - Set temperature (temperature factor) to v\n";
 
-static char *memtestFlags = "(additional options for memtest)  [-m]\n";
+static char *memtestFlags = "(additional options for memtest)  [-m<v>]\n";
 static char *memtestItems = "\n\
- -m          - Memory test (Observe no growth with top. Kill when done.)\n";
+ -m          - Memory test (Observe no growth with top. Kill when done.)\n\
+ -m[v]       - Memory test - v iterations.\n";
 
 static char *configFlags = "(additional options for config)  [-cv]\n";
 static char *configItems = "\n\
  -c          - Configure plugins (Writes $prefix/lib/graphviz/config \n\
                with available plugin information.  Needs write privilege.)\n\
- -v          - Enable verbose mode \n";
+ -?          - Print usage and exit\n";
 
-void dotneato_usage(int exval)
+/* dotneato_usage:
+ * Print usage information. If GvExitOnUsage is set, exit with
+ * given exval, else return exval+1.
+ */
+int dotneato_usage(int exval)
 {
     FILE *outs;
 
@@ -86,8 +88,10 @@ void dotneato_usage(int exval)
     fputs(memtestItems, outs);
     fputs(configItems, outs);
 
-    if (exval >= 0)
+    if (GvExitOnUsage && (exval >= 0))
 	exit(exval);
+    return (exval+1);
+	
 }
 
 /* getFlagOpt:
@@ -230,13 +234,19 @@ graph_t *gvPluginsGraph(GVC_t *gvc)
     return P_graph;
 }
 
-void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
+/* dotneato_args_initialize"
+ * Scan argv[] for allowed flags.
+ * Return 0 on success; v+1 if calling function should call exit(v).
+ * If -c is set, config file is created and we exit. 
+ */
+int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 {
     char c, *rest, *layout;
     const char *val;
     int i, v, nfiles;
     unsigned char buf[SMALLBUF];
     agxbuf xb;
+    int Kflag = 0;
 
     /* establish if we are running in a CGI environment */
     HTTPServerEnVar = getenv("SERVER_NAME");
@@ -258,23 +268,6 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
     if (gvc->common.config)
 	exit (0);
 
-    layout = gvc->common.cmdname;
-    if (streq(layout, "dot_static")
-	    || streq(layout, "dot_builtins")
-	    || streq(layout, "lt-dot_builtins")
-	    || streq(layout, "")   /* when run as a process from Gvedit on Windows */
-	)
-        layout = "dot";
-    i = gvlayout_select(gvc, layout);
-    if (i == NO_SUPPORT) {
-	fprintf(stderr, "There is no layout engine support for \"%s\"\n", layout);
-        if (streq(layout, "dot"))
-	    fprintf(stderr, "Perhaps \"dot -c\" needs to be run (with installer's privileges) to register the plugins?\n");
-	else 
-	    fprintf(stderr, "Use one of:%s\n", gvplugin_list(gvc, API_layout, ""));
-	exit(1);
-    }
-
     /* feed the globals */
     Verbose = gvc->common.verbose;
     CmdName = gvc->common.cmdname;
@@ -302,7 +295,7 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 #endif
 		else {
 		    fprintf(stderr, "Missing argument for -G flag\n");
-		    dotneato_usage(1);
+		    return (dotneato_usage(1));
 		}
 		break;
 	    case 'N':
@@ -314,7 +307,7 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 #endif
 		else {
 		    fprintf(stderr, "Missing argument for -N flag\n");
-		    dotneato_usage(1);
+		    return (dotneato_usage(1));
 		}
 		break;
 	    case 'E':
@@ -326,29 +319,28 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 #endif
 		else {
 		    fprintf(stderr, "Missing argument for -E flag\n");
-		    dotneato_usage(1);
+		    return (dotneato_usage(1));
 		}
 		break;
 	    case 'T':
 		val = getFlagOpt(argc, argv, &i);
 		if (!val) {
 		    fprintf(stderr, "Missing argument for -T flag\n");
-		    dotneato_usage(1);
-		    exit(1);
+		    return (dotneato_usage(1));
 		}
 		v = gvjobs_output_langname(gvc, val);
 		if (!v) {
 		    fprintf(stderr, "Format: \"%s\" not recognized. Use one of:%s\n",
 			val, gvplugin_list(gvc, API_device, val));
-		    exit(1);
+		    if (GvExitOnUsage) exit(1);
+		    return(2);
 		}
 		break;
 	    case 'K':
 		val = getFlagOpt(argc, argv, &i);
 		if (!val) {
                     fprintf(stderr, "Missing argument for -K flag\n");
-                    dotneato_usage(1);
-                    exit(1);
+                    return (dotneato_usage(1));
                 }
                 v = gvlayout_select(gvc, val);
                 if (v == NO_SUPPORT) {
@@ -360,8 +352,10 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
                         fprintf(stderr, "Use one of:%s\n",
 				gvplugin_list(gvc, API_layout, val));
 		    }
-		    exit(1);
+		    if (GvExitOnUsage) exit(1);
+		    return(2);
                 }
+		Kflag = 1;
 		break;
 	    case 'P':
 		P_graph = gvplugin_graph(gvc);
@@ -370,13 +364,14 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		fprintf(stderr, "%s - %s version %s (%s)\n",
 			gvc->common.cmdname, gvc->common.info[0], 
 			gvc->common.info[1], gvc->common.info[2]);
-		exit(0);
+		if (GvExitOnUsage) exit(0);
+		return (1);
 		break;
 	    case 'l':
 		val = getFlagOpt(argc, argv, &i);
 		if (!val) {
 		    fprintf(stderr, "Missing argument for -l flag\n");
-		    dotneato_usage(1);
+		    return (dotneato_usage(1));
 		}
 		use_library(gvc, val);
 		break;
@@ -406,7 +401,7 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 			fprintf(stderr,
 				"Invalid parameter \"%s\" for -s flag\n",
 				rest);
-			dotneato_usage(1);
+			return (dotneato_usage(1));
 		    }
 		} else
 		    PSinputscale = POINTS_PER_INCH;
@@ -418,18 +413,41 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		Y_invert = TRUE;
 		break;
 	    case '?':
-		dotneato_usage(0);
+		return (dotneato_usage(0));
 		break;
 	    default:
-		fprintf(stderr, "%s: option -%c unrecognized\n\n", gvc->common.cmdname,
+		agerr(AGERR, "%s: option -%c unrecognized\n\n", gvc->common.cmdname,
 			c);
-		dotneato_usage(1);
+		return (dotneato_usage(1));
 	    }
 	} else if (argv[i])
 	    gvc->input_filenames[nfiles++] = argv[i];
     }
     agxbfree (&xb);
 
+    /* if no -K, use cmd name to set layout type */
+    if (!Kflag) {
+	layout = gvc->common.cmdname;
+	if (streq(layout, "dot_static")
+	    || streq(layout, "dot_builtins")
+	    || streq(layout, "lt-dot")
+	    || streq(layout, "lt-dot_builtins")
+	    || streq(layout, "")   /* when run as a process from Gvedit on Windows */
+	)
+            layout = "dot";
+	i = gvlayout_select(gvc, layout);
+	if (i == NO_SUPPORT) {
+	    fprintf(stderr, "There is no layout engine support for \"%s\"\n", layout);
+            if (streq(layout, "dot"))
+		fprintf(stderr, "Perhaps \"dot -c\" needs to be run (with installer's privileges) to register the plugins?\n");
+	    else 
+		fprintf(stderr, "Use one of:%s\n", gvplugin_list(gvc, API_layout, ""));
+
+	    if (GvExitOnUsage) exit(1);
+	    return(2);
+	}
+    }
+
     /* if no -Txxx, then set default format */
     if (!gvc->jobs || !gvc->jobs->output_langname) {
 	v = gvjobs_output_langname(gvc, "dot");
@@ -437,7 +455,7 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 //	assert(v);  /* "dot" should always be available as an output format */
 		fprintf(stderr,
 			"Unable to find even the default \"-Tdot\" renderer.  Has the config\nfile been generated by running \"dot -c\" with installer's priviledges?\n");
-			exit(1);
+			return(2);
 	}
     }
 
@@ -449,6 +467,7 @@ void dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
     if (!(agfindnodeattr(agprotograph(), "label")))
 	agnodeattr(NULL, "label", NODENAME_ESC);
 #endif
+    return 0;
 }
 
 /* getdoubles2ptf:
@@ -551,6 +570,7 @@ graph_t *gvNextInputGraph(GVC_t *gvc)
     graph_t *g = NULL;
     static char *fn;
     static FILE *fp;
+    static FILE *oldfp;
     static int fidx, gidx;
 
     while (!g) {
@@ -568,7 +588,10 @@ graph_t *gvNextInputGraph(GVC_t *gvc)
 	}
 	if (fp == NULL)
 	    break;
-	agsetfile(fn ? fn : "<stdin>");
+	if (oldfp != fp) {
+	    agsetfile(fn ? fn : "<stdin>");
+	    oldfp = fp;
+	}
 #ifdef EXPERIMENTAL_MYFGETS
 	g = agread_usergets(fp, myfgets);
 #else
@@ -672,7 +695,6 @@ void graph_init(graph_t * g, boolean use_rankdir)
     static char *fontnamenames[] = {"gd","ps","svg", NULL};
     static int fontnamecodes[] = {NATIVEFONTS,PSFONTS,SVGFONTS,-1};
     int rankdir;
-
     GD_drawing(g) = NEW(layout_t);
 
     /* set this up fairly early in case any string sizes are needed */
@@ -693,6 +715,12 @@ void graph_init(graph_t * g, boolean use_rankdir)
 
     GD_charset(g) = findCharset (g);
 
+    if (!HTTPServerEnVar) {
+	Gvimagepath = agget (g, "imagepath");
+	if (!Gvimagepath)
+	    Gvimagepath = Gvfilepath;
+    }
+
     GD_drawing(g)->quantum =
 	late_double(g, agfindgraphattr(g, "quantum"), 0.0, 0.0);
 
@@ -758,6 +786,7 @@ void graph_init(graph_t * g, boolean use_rankdir)
     p = agget(g, "concentrate");
     Concentrate = mapbool(p);
     State = GVBEGIN;
+    EdgeLabelsDone = 0;
 
     GD_drawing(g)->dpi = 0.0;
     if (((p = agget(g, "dpi")) && p[0])
@@ -768,6 +797,10 @@ void graph_init(graph_t * g, boolean use_rankdir)
 
     Initial_dist = MYHUGE;
 
+    G_ordering = agfindgraphattr(g, "ordering");
+    G_gradientangle = agfindgraphattr(g,"gradientangle");
+    G_margin = agfindgraphattr(g, "margin");
+
     /* initialize nodes */
     N_height = agfindnodeattr(g, "height");
     N_width = agfindnodeattr(g, "width");
@@ -779,9 +812,17 @@ void graph_init(graph_t * g, boolean use_rankdir)
     N_fontname = agfindnodeattr(g, "fontname");
     N_fontcolor = agfindnodeattr(g, "fontcolor");
     N_label = agfindnodeattr(g, "label");
+    if (!N_label)
+#ifdef WITH_CGRAPH
+	N_label = agattr(g, AGNODE, "label", NODENAME_ESC);
+#else
+	N_label = agnodeattr(g, "label", NODENAME_ESC);
+#endif
     N_xlabel = agfindnodeattr(g, "xlabel");
     N_showboxes = agfindnodeattr(g, "showboxes");
     N_penwidth = agfindnodeattr(g, "penwidth");
+    N_ordering = agfindnodeattr(g, "ordering");
+    N_margin = agfindnodeattr(g, "margin");
     /* attribs for polygon shapes */
     N_sides = agfindnodeattr(g, "sides");
     N_peripheries = agfindnodeattr(g, "peripheries");
@@ -796,10 +837,12 @@ void graph_init(graph_t * g, boolean use_rankdir)
     N_comment = agfindnodeattr(g, "comment");
     N_vertices = agfindnodeattr(g, "vertices");
     N_z = agfindnodeattr(g, "z");
+    N_gradientangle = agfindnodeattr(g,"gradientangle");
 
     /* initialize edges */
     E_weight = agfindedgeattr(g, "weight");
     E_color = agfindedgeattr(g, "color");
+    E_fillcolor = agfindedgeattr(g, "fillcolor");
     E_fontsize = agfindedgeattr(g, "fontsize");
     E_fontname = agfindedgeattr(g, "fontname");
     E_fontcolor = agfindedgeattr(g, "fontcolor");
@@ -832,12 +875,19 @@ void graph_init(graph_t * g, boolean use_rankdir)
 
     /* background */
     GD_drawing(g)->xdots = init_xdot (g);
+
+    /* initialize id, if any */
+
+    if ((p = agget(g, "id")) && *p)
+	GD_drawing(g)->id = strdup_and_subst_obj(p, g);
 }
 
 void graph_cleanup(graph_t *g)
 {
     if (GD_drawing(g)->xdots)
 	freeXDot ((xdot*)GD_drawing(g)->xdots);
+    if (GD_drawing(g)->id)
+	free (GD_drawing(g)->id);
     free(GD_drawing(g));
     GD_drawing(g) = NULL;
     free_label(GD_label(g));
