@@ -1,18 +1,15 @@
-/* $Id: output.c,v 1.71 2009/10/15 17:28:19 erg Exp $ $Revision: 1.71 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #include "render.h"
 #include "agxbuf.h"
@@ -22,8 +19,13 @@
 #define YDIR(y) (Y_invert ? (Y_off - (y)) : (y))
 #define YFDIR(y) (Y_invert ? (YF_off - (y)) : (y))
 
-int Y_off;           /* ymin + ymax */
-double YF_off;       /* Y_off in inches */
+static double Y_off;        /* ymin + ymax */
+static double YF_off;       /* Y_off in inches */
+
+double yDir (double y)
+{
+    return YDIR(y);
+}
 
 #ifdef WITH_CGRAPH
 static int (*putstr) (void *chan, const char *str);
@@ -80,7 +82,7 @@ static void printpoint(FILE * f, pointf p)
 static void setYInvert(graph_t * g)
 {
     if (Y_invert) {
-	Y_off = ROUND(GD_bb(g).UR.y + GD_bb(g).LL.y);
+	Y_off = GD_bb(g).UR.y + GD_bb(g).LL.y;
 	YF_off = PS2INCH(Y_off);
     }
 }
@@ -125,6 +127,7 @@ void write_plain(GVJ_t * job, graph_t * g, FILE * f, boolean extend)
     bezier bz;
     pointf pt;
     char *lbl;
+    char* fillcolor;
 
 #ifdef WITH_CGRAPH
     putstr = g->clos->disc.io->putstr;
@@ -155,7 +158,10 @@ void write_plain(GVJ_t * job, graph_t * g, FILE * f, boolean extend)
 	printstring(f, " ", late_nnstring(n, N_style, "solid"));
 	printstring(f, " ", ND_shape(n)->name);
 	printstring(f, " ", late_nnstring(n, N_color, DEFAULT_COLOR));
-	printstring(f, " ", late_nnstring(n, N_fillcolor, DEFAULT_FILL));
+	fillcolor = late_nnstring(n, N_fillcolor, "");
+        if (fillcolor[0] == '\0')
+	    fillcolor = late_nnstring(n, N_color, DEFAULT_FILL);
+	printstring(f, " ", fillcolor);
 	agputc('\n', f);
     }
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -217,9 +223,9 @@ static void set_record_rects(node_t * n, field_t * f, agxbuf * xb)
     if (f->n_flds == 0) {
 	sprintf(buf, "%.5g,%.5g,%.5g,%.5g ",
 		f->b.LL.x + ND_coord(n).x,
-		YFDIR(f->b.LL.y + ND_coord(n).y),
+		YDIR(f->b.LL.y + ND_coord(n).y),
 		f->b.UR.x + ND_coord(n).x,
-		YFDIR(f->b.UR.y + ND_coord(n).y));
+		YDIR(f->b.UR.y + ND_coord(n).y));
 	agxbput(xb, buf);
     }
     for (i = 0; i < f->n_flds; i++)
@@ -232,8 +238,8 @@ static void rec_attach_bb(graph_t * g, Agsym_t* bbsym)
     char buf[BUFSIZ];
     pointf pt;
 
-    sprintf(buf, "%.5g,%.5g,%.5g,%.5g", GD_bb(g).LL.x, YFDIR(GD_bb(g).LL.y),
-	    GD_bb(g).UR.x, YFDIR(GD_bb(g).UR.y));
+    sprintf(buf, "%.5g,%.5g,%.5g,%.5g", GD_bb(g).LL.x, YDIR(GD_bb(g).LL.y),
+	    GD_bb(g).UR.x, YDIR(GD_bb(g).UR.y));
 #ifndef WITH_CGRAPH
     agxset(g, bbsym->index, buf);
 #else
@@ -241,8 +247,13 @@ static void rec_attach_bb(graph_t * g, Agsym_t* bbsym)
 #endif
     if (GD_label(g) && GD_label(g)->text[0]) {
 	pt = GD_label(g)->pos;
-	sprintf(buf, "%.5g,%.5g", pt.x, YFDIR(pt.y));
+	sprintf(buf, "%.5g,%.5g", pt.x, YDIR(pt.y));
 	agset(g, "lp", buf);
+	pt = GD_label(g)->dimen;
+	sprintf(buf, "%.2f", PS2INCH(pt.x));
+	agset (g, "lwidth", buf);
+	sprintf(buf, "%.2f", PS2INCH(pt.y));
+	agset (g, "lheight", buf);
     }
     for (c = 1; c <= GD_n_cluster(g); c++)
 	rec_attach_bb(GD_clust(g)[c], bbsym);
@@ -284,10 +295,17 @@ void attach_attrs_and_arrows(graph_t* g, int* sp, int* ep)
 	safe_dcl(g, g->proto->e, "tail_lp", "", agedgeattr);
     if (GD_label(g)) {
 	safe_dcl(g, g, "lp", "", agraphattr);
+	safe_dcl(g, g, "lwidth", "", agraphattr);
+	safe_dcl(g, g, "lheight", "", agraphattr);
 	if (GD_label(g)->text[0]) {
 	    ptf = GD_label(g)->pos;
-	    sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+	    sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 	    agset(g, "lp", buf);
+	    ptf = GD_label(g)->dimen;
+	    sprintf(buf, "%.2f", PS2INCH(ptf.x));
+	    agset(g, "lwidth", buf);
+	    sprintf(buf, "%.2f", PS2INCH(ptf.y));
+	    agset(g, "lheight", buf);
 	}
     }
     bbsym = safe_dcl(g, g, "bb", "", agraphattr);
@@ -309,21 +327,36 @@ void attach_attrs_and_arrows(graph_t* g, int* sp, int* ep)
 	safe_dcl(g, AGEDGE, "tail_lp", "");
     if (GD_label(g)) {
 	safe_dcl(g, AGRAPH, "lp", "");
+	safe_dcl(g, AGRAPH, "lwidth", "");
+	safe_dcl(g, AGRAPH, "lheight", "");
 	if (GD_label(g)->text[0]) {
 	    ptf = GD_label(g)->pos;
-	    sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+	    sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 	    agset(g, "lp", buf);
+	    ptf = GD_label(g)->dimen;
+	    sprintf(buf, "%.2f", PS2INCH(ptf.x));
+	    agset(g, "lwidth", buf);
+	    sprintf(buf, "%.2f", PS2INCH(ptf.y));
+	    agset(g, "lheight", buf);
 	}
     }
     bbsym = safe_dcl(g, AGRAPH, "bb", "");
 #endif
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (dim3) {
+	    int k;
+
 	    sprintf(buf, "%.5g,%.5g,%.5g", ND_coord(n).x, YDIR(ND_coord(n).y), POINTS_PER_INCH*(ND_pos(n)[2]));
+	    agxbput (&xb, buf);
+	    for (k = 3; k < GD_odim(g); k++) {
+		sprintf(buf, ",%.5g", POINTS_PER_INCH*(ND_pos(n)[k]));
+		agxbput (&xb, buf);
+	    }
+	    agset(n, "pos", agxbuse(&xb));
 	} else {
 	    sprintf(buf, "%.5g,%.5g", ND_coord(n).x, YDIR(ND_coord(n).y));
+	    agset(n, "pos", buf);
 	}
-	agset(n, "pos", buf);
 	sprintf(buf, "%.5g", PS2INCH(ND_ht(n)));
 #ifndef WITH_CGRAPH
 	agxset(n, N_height->index, buf);
@@ -334,9 +367,9 @@ void attach_attrs_and_arrows(graph_t* g, int* sp, int* ep)
 	sprintf(buf, "%.5g", PS2INCH(ND_lw(n) + ND_rw(n)));
 	agxset(n, N_width, buf);
 #endif
-	if (ND_xlabel(n)) {
+	if (ND_xlabel(n) && ND_xlabel(n)->set) {
 	    ptf = ND_xlabel(n)->pos;
-	    sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+	    sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 	    agset(n, "xlp", buf);
 	}
 	if (strcmp(ND_shape(n)->name, "record") == 0) {
@@ -391,38 +424,38 @@ void attach_attrs_and_arrows(graph_t* g, int* sp, int* ep)
 			s_arrows = 1;
 			sprintf(buf, "s,%.5g,%.5g ",
 				ED_spl(e)->list[i].sp.x,
-				YFDIR(ED_spl(e)->list[i].sp.y));
+				YDIR(ED_spl(e)->list[i].sp.y));
 			agxbput(&xb, buf);
 		    }
 		    if (ED_spl(e)->list[i].eflag) {
 			e_arrows = 1;
 			sprintf(buf, "e,%.5g,%.5g ",
 				ED_spl(e)->list[i].ep.x,
-				YFDIR(ED_spl(e)->list[i].ep.y));
+				YDIR(ED_spl(e)->list[i].ep.y));
 			agxbput(&xb, buf);
 		    }
 		    for (j = 0; j < ED_spl(e)->list[i].size; j++) {
 			if (j > 0)
 			    agxbputc(&xb, ' ');
 			ptf = ED_spl(e)->list[i].list[j];
-			sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+			sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 			agxbput(&xb, buf);
 		    }
 		}
 		agset(e, "pos", agxbuse(&xb));
 		if (ED_label(e)) {
 		    ptf = ED_label(e)->pos;
-		    sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+		    sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 		    agset(e, "lp", buf);
 		}
-		if (ED_xlabel(e)) {
+		if (ED_xlabel(e) && ED_xlabel(e)->set) {
 		    ptf = ED_xlabel(e)->pos;
-		    sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+		    sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 		    agset(e, "xlp", buf);
 		}
 		if (ED_head_label(e)) {
 		    ptf = ED_head_label(e)->pos;
-		    sprintf(buf, "%.5g,%.5g", ptf.x, YFDIR(ptf.y));
+		    sprintf(buf, "%.5g,%.5g", ptf.x, YDIR(ptf.y));
 		    agset(e, "head_lp", buf);
 		}
 		if (ED_tail_label(e)) {
@@ -450,10 +483,3 @@ void attach_attrs(graph_t * g)
     attach_attrs_and_arrows (g, &s, &e);
 }
 
-void output_point(agxbuf *xbuf, pointf p)
-{
-    char buf[BUFSIZ];
-    sprintf(buf, "%d %d ", ROUND(p.x), YDIR(ROUND(p.y)));
-    agxbput(xbuf, buf);
-}
-
