@@ -1,18 +1,15 @@
-/* $Id: aspect.c,v 1.7 2009/06/03 01:10:52 ellson Exp $ $Revision: 1.7 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/***********************************************************
- *      This software is part of the graphviz package      *
- *                http://www.graphviz.org/                 *
- *                                                         *
- *            Copyright (c) 1994-2008 AT&T Corp.           *
- *                and is licensed under the                *
- *            Common Public License, Version 1.0           *
- *                      by AT&T Corp.                      *
- *                                                         *
- *        Information and Software Systems Research        *
- *              AT&T Research, Florham Park NJ             *
- **********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #include "dot.h"
 
@@ -27,6 +24,9 @@
  *   - Provide algorithms for aspect ratios < 1
  */
 
+#define MIN_AR 1.0
+#define MAX_AR 20.0
+#define DEF_PASSES 5
 #define DPI 72
 
 /*
@@ -1788,10 +1788,11 @@ void rank3(graph_t * g, aspect_t * asp)
     Agnode_t *n;
     int i;
     int iterations = asp->nextIter;
+    double lastAR = MAXDOUBLE;
 
     computeNodeGroups(g);	/* groups of UF DS nodes */
 
-    for (i = 0; i < iterations || iterations == -1; i++) {
+    for (i = 0; (i < iterations) || (iterations == -1); i++) {
 	/* initialize all ranks to be 0 */
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    ND_rank(n) = 0;
@@ -1817,12 +1818,14 @@ void rank3(graph_t * g, aspect_t * asp)
 	    break;
 	} else
 #endif
-	if (iterations == -1 && asp->combiAR <= asp->targetAR) {
+        /* Success or if no improvement */
+	if ((asp->combiAR <= asp->targetAR) || ((iterations == -1) && (lastAR <= asp->combiAR))) {
 	    asp->prevIterations = asp->curIterations;
 	    asp->curIterations = i;
 
 	    break;
 	}
+	lastAR = asp->combiAR;
 	/* Apply the FFDH algorithm to reduce the aspect ratio; */
 	applyPacking2(g);
     }
@@ -1970,3 +1973,31 @@ void init_UF_size(graph_t * g)
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
 	ND_UF_size(n) = 0;
 }
+
+aspect_t*
+setAspect (Agraph_t * g, aspect_t* adata)
+{
+    double rv;
+    char *p;
+    int r, passes = DEF_PASSES;
+
+    p = agget (g, "aspect");
+
+    if (!p || ((r = sscanf (p, "%lf,%d", &rv, &passes)) <= 0)) {
+	adata->nextIter = 0;
+	adata->badGraph = 0;
+	return NULL;
+    }
+    
+    if (rv < MIN_AR) rv = MIN_AR;
+    else if (rv > MAX_AR) rv = MAX_AR;
+    adata->targetAR = rv;
+    adata->nextIter = -1;
+    adata->nPasses = passes;
+    adata->badGraph = 0;
+
+    if (Verbose) 
+        fprintf(stderr, "Target AR = %g\n", adata->targetAR);
+
+    return adata;
+}
