@@ -1,18 +1,15 @@
-/* $Id: class2.c,v 1.8 2008/10/23 20:23:55 ellson Exp $ $Revision: 1.8 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 /* classify edges for mincross/nodepos/splines, using given ranks */
@@ -281,7 +278,11 @@ void class2(graph_t * g)
 	    else {
 		/*other_edge(e); */
 		/* avoid when opp==e in undirected graph */
+#ifndef WITH_CGRAPH
 		if ((opp = agfindedge(g, aghead(e), agtail(e))) && (opp != e)) {
+#else
+		if ((opp = agfindedge(g, aghead(e), agtail(e))) && (aghead(opp) != aghead(e))) {
+#endif /* WITH_CGRAPH */
 		    /* shadows a forward edge */
 		    if (ED_to_virt(opp) == NULL)
 			make_chain(g, agtail(opp), aghead(opp), opp);
