@@ -1,18 +1,15 @@
-/* $Id: fdpinit.c,v 1.24 2009/06/11 15:55:20 erg Exp $ $Revision: 1.24 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 /* fdpinit.c:
@@ -89,11 +86,20 @@ static void init_edge(edge_t * e, attrsym_t * E_len)
     agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//node custom data
 #endif /* WITH_CGRAPH */
     ED_factor(e) = late_double(e, E_weight, 1.0, 0.0);
-    ED_dist(e) = late_double(e, E_len, fdp_parms.K, 0.0);
+    ED_dist(e) = late_double(e, E_len, fdp_parms->K, 0.0);
 
     common_init_edge(e);
 }
 
+#ifdef WITH_CGRAPH
+static void init_node(node_t * n)
+{
+    common_init_node(n);
+    ND_pos(n) = N_NEW(GD_ndim(agraphof(n)), double);
+    gv_nodesize(n, GD_flip(agraphof(n)));
+}
+#endif
+
 void fdp_init_node_edge(graph_t * g)
 {
     attrsym_t *E_len;
@@ -101,20 +107,27 @@ void fdp_init_node_edge(graph_t * g)
     edge_t *e;
     int nn;
     int i;
-    ndata* alg;
+    /* ndata* alg; */
 
+#ifdef WITH_CGRAPH
+    aginit(g, AGNODE, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+#endif
     processClusterEdges(g);
 
     /* Get node count after processClusterEdges(), as this function may
      * add new nodes.
      */
     nn = agnnodes(g);
-    alg = N_NEW(nn, ndata);
+    /* alg = N_NEW(nn, ndata); */
     GD_neato_nlist(g) = N_NEW(nn + 1, node_t *);
 
     for (i = 0, n = agfstnode(g); n; n = agnxtnode(g, n)) {
+#ifdef WITH_CGRAPH
+	init_node (n);
+#else
 	neato_init_node (n);
-	ND_alg(n) = alg + i;
+#endif
+	/* ND_alg(n) = alg + i; */
 	GD_neato_nlist(g)[i] = n;
 	ND_id(n) = i++;
     }
@@ -152,7 +165,7 @@ static void cleanup_subgs(graph_t * g)
 #ifndef WITH_CGRAPH
 	memset(&(g->u), 0, sizeof(Agraphinfo_t));
 #else /* WITH_CGRAPH */
-	agclean(g, AGRAPH , "Agraphinfo_t");				
+	agdelrec(g, "Agraphinfo_t");				
 #endif /* WITH_CGRAPH */
 }
 
