@@ -57,14 +57,14 @@ extern double drand48(void);
 #include "tlayout.h"
 #include "globals.h"
 
-#define D_useGrid   (fdp_parms.useGrid)
-#define D_useNew    (fdp_parms.useNew)
-#define D_numIters  (fdp_parms.numIters)
-#define D_unscaled  (fdp_parms.unscaled)
-#define D_C         (fdp_parms.C)
-#define D_Tfact     (fdp_parms.Tfact)
-#define D_K         (fdp_parms.K)
-#define D_T0        (fdp_parms.T0)
+#define D_useGrid   (fdp_parms->useGrid)
+#define D_useNew    (fdp_parms->useNew)
+#define D_numIters  (fdp_parms->numIters)
+#define D_unscaled  (fdp_parms->unscaled)
+#define D_C         (fdp_parms->C)
+#define D_Tfact     (fdp_parms->Tfact)
+#define D_K         (fdp_parms->K)
+#define D_T0        (fdp_parms->T0)
 
   /* Actual parameters used; initialized using fdp_parms, then possibly
    * updated with graph-specific values.
@@ -152,7 +152,8 @@ static int init_params(graph_t * g, xparams * xpms)
 #ifdef DEBUG
 	if (Verbose) {
 	    prIndent();
-	    fprintf(stderr, "tlayout %s(%s) : T0 %f\n", g->name, GORIG(g->root)->name, T_T0);
+	    fprintf(stderr, "tlayout %s", agnameof(g));
+	    fprintf(stderr, "(%s) : T0 %f\n", agnameof(GORIG(g->root)), T_T0);
 	}
 #endif
 	ret = 1;
@@ -225,7 +226,7 @@ void fdp_initParams(graph_t * g)
 	prIndent();
 	fprintf(stderr,
 		"Params %s : K %f T0 %f Tfact %f maxIters %d unscaled %d\n",
-		g->name, 
+		agnameof(g),
                 T_K, T_T0, T_Tfact, T_maxIters, T_unscaled);
     }
 #endif
