@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.6.1.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +30,15 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_Y_TAB_H
+# define YY_Y_TAB_H
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int agdebug;
+#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -63,13 +70,11 @@
 
 
 
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 1676 of yacc.c  */
-#line 308 "../../lib/graph/parser.y"
+/* Line 2049 of yacc.c  */
+#line 309 "../../lib/graph/parser.y"
 
 			int					i;
 			char				*str;
@@ -77,9 +82,8 @@ typedef union YYSTYPE
 			struct Agnode_t		*n;
 
 
-
-/* Line 1676 of yacc.c  */
-#line 83 "y.tab.h"
+/* Line 2049 of yacc.c  */
+#line 87 "y.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define agstype YYSTYPE /* obsolescent; will be withdrawn */
@@ -88,4 +92,18 @@ typedef union YYSTYPE
 
 extern YYSTYPE aglval;
 
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int agparse (void *YYPARSE_PARAM);
+#else
+int agparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int agparse (void);
+#else
+int agparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
+#endif /* !YY_Y_TAB_H  */
