@@ -1,4 +1,4 @@
- /* $Id: gvc.c,v 1.67 2011/03/09 16:24:19 arif Exp $ $Revision: 1.67 $ */
+ /* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -22,6 +22,9 @@
 #include "gvcproc.h"
 #include "gvconfig.h"
 #include "gvio.h"
+#ifdef WITH_CGRAPH
+#include <stdlib.h>
+#endif
 
 #ifdef WIN32 /*dependencies*/
 #ifdef WITH_CGRAPH
@@ -249,6 +252,14 @@ int gvRenderData(GVC_t *gvc, graph_t *g, const char *format, char **result, unsi
     return rc;
 }
 
+/* gvFreeRenderData:
+ * Utility routine to free memory allocated in gvRenderData, as the application code may use
+ * a different runtime library.
+ */
+void gvFreeRenderData (char* data)
+{
+    free (data);
+}
 
 void gvAddLibrary(GVC_t *gvc, gvplugin_library_t *lib)
 {
