@@ -1,18 +1,15 @@
-/* $Id: gvcproc.h,v 1.50 2009/02/18 13:36:50 glenlow Exp $ $Revision: 1.50 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 /* This is the public header for the callers of libgvc */
 
@@ -68,7 +65,7 @@
     extern usershape_t *gvusershape_find(char *name);
 
 /* device */
-    extern void gvdevice_initialize(GVJ_t * job);
+    extern int gvdevice_initialize(GVJ_t * job);
     extern void gvdevice_format(GVJ_t * job);
     extern void gvdevice_finalize(GVJ_t * job);
 
@@ -77,7 +74,7 @@
     extern pointf gvrender_ptf(GVJ_t *job, pointf p);
     extern pointf* gvrender_ptf_A(GVJ_t *job, pointf *af, pointf *AF, int n);
 
-    extern void gvrender_begin_job(GVJ_t * job);
+    extern int gvrender_begin_job(GVJ_t * job);
     extern void gvrender_end_job(GVJ_t * job);
     extern int gvrender_select(GVJ_t * job, const char *lang);
     extern int gvrender_features(GVJ_t * job);
@@ -97,23 +94,21 @@
     extern void gvrender_end_node(GVJ_t * job);
     extern void gvrender_begin_edge(GVJ_t * job, edge_t * e);
     extern void gvrender_end_edge(GVJ_t * job);
-    extern void gvrender_begin_context(GVJ_t * job);
-    extern void gvrender_end_context(GVJ_t * job);
     extern void gvrender_begin_anchor(GVJ_t * job,
 		char *href, char *tooltip, char *target, char *id);
     extern void gvrender_end_anchor(GVJ_t * job);
     extern void gvrender_begin_label(GVJ_t * job, label_type type);
     extern void gvrender_end_label(GVJ_t * job);
-    extern void gvrender_set_font(GVJ_t * job, char *fontname,
-			double fontsize);
     extern void gvrender_textpara(GVJ_t * job, pointf p, textpara_t * para);
     extern void gvrender_set_pencolor(GVJ_t * job, char *name);
     extern void gvrender_set_penwidth(GVJ_t * job, double penwidth);
     extern void gvrender_set_fillcolor(GVJ_t * job, char *name);
+    extern void gvrender_set_gradient_vals (GVJ_t * job, char *stopcolor, int angle, float frac);
+
     extern void gvrender_set_style(GVJ_t * job, char **s);
-    extern void gvrender_ellipse(GVJ_t * job, pointf * AF, int n, boolean filled);
-    extern void gvrender_polygon(GVJ_t * job, pointf * AF, int n, boolean filled);
-    extern void gvrender_box(GVJ_t * job, boxf BF, boolean filled);
+    extern void gvrender_ellipse(GVJ_t * job, pointf * AF, int n, int filled);
+    extern void gvrender_polygon(GVJ_t* job, pointf* af, int n, int filled);
+    extern void gvrender_box(GVJ_t * job, boxf BF, int filled);
     extern void gvrender_beziercurve(GVJ_t * job, pointf * AF, int n,
 			int arrow_at_start, int arrow_at_end, boolean filled);
     extern void gvrender_polyline(GVJ_t * job, pointf * AF, int n);
