@@ -1,18 +1,15 @@
-/* $Id: gvevent.c,v 1.65 2008/10/23 21:09:38 ellson Exp $ $Revision: 1.65 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -672,9 +669,9 @@ static void gvevent_read (GVJ_t * job, const char *filename, const char *layout)
 #ifndef WITH_CGRAPH
 	g = agopen("G", AGDIGRAPH);
 #else /* WITH_CGRAPH */
-		agopen("G", Agdirected, NIL(Agdisc_t *));
+	g = agopen("G", Agdirected, NIL(Agdisc_t *));
 #endif /* WITH_CGRAPH */
-	job->output_filename = "new.dot";
+	job->output_filename = "new.gv";
     }
     else {
 	f = fopen(filename, "r");
@@ -684,11 +681,13 @@ static void gvevent_read (GVJ_t * job, const char *filename, const char *layout)
 	g = agread(f);
 #else /* WITH_CGRAPH */
 	g = agread(f,NIL(Agdisc_t *));
+
 #endif /* WITH_CGRAPH */
 	fclose(f);
     }
     if (!g)
 	return;   /* FIXME - need some error handling */
+
     if (gvc->g) {
 	gvle = gvc->layout.engine;
 	if (gvle && gvle->cleanup)
@@ -696,6 +695,12 @@ static void gvevent_read (GVJ_t * job, const char *filename, const char *layout)
 	graph_cleanup(gvc->g);
 	agclose(gvc->g);
     }
+
+#ifdef WITH_CGRAPH
+    aginit (g, AGRAPH, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+    aginit (g, AGNODE, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+    aginit (g, AGEDGE, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+#endif
     gvc->g = g;
     GD_gvc(g) = gvc;
     gvLayout(gvc, g, layout);
