@@ -1,4 +1,4 @@
-/* $Id: gvplugin.c,v 1.91 2011/02/26 19:56:38 erg Exp $ $Revision: 1.91 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -35,6 +35,10 @@
 extern int strcasecmp(const char *s1, const char *s2);
 #endif
 
+#ifdef WIN32
+#define strdup(x) _strdup(x)
+#endif
+
 /*
  * Define an apis array of name strings using an enumerated api_t as index.
  * The enumerated type is defined gvplugin.h.  The apis array is
@@ -362,17 +366,17 @@ char *gvplugin_list(GVC_t * gvc, api_t api, const char *str)
     if (api < 0)
 	return NULL;
 
+    /* check for valid str */
+    if (! str)
+	return NULL;
+
     if (first) {
 	agxbinit(&xb, 0, 0);
 	first = 0;
     }
 
     /* does str have a :path modifier? */
-#ifdef WIN32
-    s = _strdup(str);
-#else
     s = strdup(str);
-#endif
     p = strchr(s, ':');
     if (p)
 	*p++ = '\0';
