@@ -1,18 +1,15 @@
-/* $Id: gvusershape.c,v 1.51 2009/06/03 01:10:53 ellson Exp $ $Revision: 1.51 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -20,8 +17,22 @@
 
 #include <stddef.h>
 #include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
 #include <errno.h>
 
+#ifdef WIN32
+#include <windows.h>
+#define GLOB_NOSPACE    1   /* Ran out of memory.  */
+#define GLOB_ABORTED    2   /* Read error.  */
+#define GLOB_NOMATCH    3   /* No matches found.  */
+#define GLOB_NOSORT     4
+#define DMKEY "Software\\Microsoft" //key to look for library dir
+#include "regex_win32.h"
+#else
+#include <regex.h>
+#endif
+
 #include "types.h"
 #include "logic.h"
 #include "memory.h"
@@ -31,7 +42,8 @@
 #include "utils.h"
 #include "gvplugin_loadimage.h"
 
-
+extern char *Gvimagepath;
+extern char *HTTPServerEnVar;
 extern shape_desc *find_user_shape(const char *);
 
 static Dict_t *ImageDict;
@@ -54,17 +66,20 @@ typedef struct {
 #define EPS_MAGIC  "\xC5\xD0\xD3\xC6"
 #define XML_MAGIC  "<?xml"
 #define SVG_MAGIC  "<svg"
+#define RIFF_MAGIC  "RIFF"
+#define WEBP_MAGIC  "WEBP"
 
 static knowntype_t knowntypes[] = {
-    { PNG_MAGIC,  sizeof(PNG_MAGIC)-1,  FT_PNG,  "png",  },
-    { PS_MAGIC,   sizeof(PS_MAGIC)-1,   FT_PS,   "ps",   },
-    { BMP_MAGIC,  sizeof(BMP_MAGIC)-1,  FT_BMP,  "bmp",  },
-    { GIF_MAGIC,  sizeof(GIF_MAGIC)-1,  FT_GIF,  "gif",  },
-    { JPEG_MAGIC, sizeof(JPEG_MAGIC)-1, FT_JPEG, "jpeg", },
-    { PDF_MAGIC,  sizeof(PDF_MAGIC)-1,  FT_PDF,  "pdf",  },
-    { EPS_MAGIC,  sizeof(EPS_MAGIC)-1,  FT_EPS,  "eps",  },
+    { PNG_MAGIC,  sizeof(PNG_MAGIC)-1,   FT_PNG,  "png",  },
+    { PS_MAGIC,   sizeof(PS_MAGIC)-1,    FT_PS,   "ps",   },
+    { BMP_MAGIC,  sizeof(BMP_MAGIC)-1,   FT_BMP,  "bmp",  },
+    { GIF_MAGIC,  sizeof(GIF_MAGIC)-1,   FT_GIF,  "gif",  },
+    { JPEG_MAGIC, sizeof(JPEG_MAGIC)-1,  FT_JPEG, "jpeg", },
+    { PDF_MAGIC,  sizeof(PDF_MAGIC)-1,   FT_PDF,  "pdf",  },
+    { EPS_MAGIC,  sizeof(EPS_MAGIC)-1,   FT_EPS,  "eps",  },
 /*    { SVG_MAGIC,  sizeof(SVG_MAGIC)-1,  FT_SVG,  "svg",  },  - viewers expect xml preamble */
-    { XML_MAGIC,  sizeof(XML_MAGIC)-1,  FT_XML,  "xml",  },
+    { XML_MAGIC,  sizeof(XML_MAGIC)-1,   FT_XML,  "xml",  },
+    { RIFF_MAGIC,  sizeof(RIFF_MAGIC)-1, FT_RIFF, "riff", },
 };
 
 static int imagetype (usershape_t *us)
@@ -78,15 +93,23 @@ static int imagetype (usershape_t *us)
 	    if (!memcmp (header, knowntypes[i].template, knowntypes[i].size)) {
 	        us->stringtype = knowntypes[i].stringtype;
 		us->type = knowntypes[i].type;
-		if (us->type != FT_XML)
-		    return us->type;
-		/* check for SVG in case of XML */
-		while (fgets(line, sizeof(line), us->f) != NULL) {
-		    if (!memcmp(line, SVG_MAGIC, sizeof(SVG_MAGIC)-1)) {
-    			us->stringtype = "svg";
-			return (us->type = FT_SVG);
+		if (us->type == FT_XML) {
+		    /* check for SVG in case of XML */
+		    while (fgets(line, sizeof(line), us->f) != NULL) {
+		        if (!memcmp(line, SVG_MAGIC, sizeof(SVG_MAGIC)-1)) {
+    			    us->stringtype = "svg";
+			    return (us->type = FT_SVG);
+		        }
 		    }
 		}
+	    	else if (us->type == FT_RIFF) {
+		    /* check for WEBP in case of RIFF */
+		    if (!memcmp(header+8, WEBP_MAGIC, sizeof(WEBP_MAGIC)-1)) {
+    			us->stringtype = "webp";
+			return (us->type = FT_WEBP);
+		    }
+		}
+		return us->type;
 	    }
         }
     }
@@ -143,33 +166,71 @@ static unsigned int svg_units_convert(double n, char *u)
     return 0;
 }
 
+static char* svg_attr_value_re = "([a-z][a-zA-Z]*)=\"([^\"]*)\"";
+static regex_t re, *pre = NULL;
+
 static void svg_size (usershape_t *us)
 {
     unsigned int w = 0, h = 0;
-    double n;
+    double n, x0, y0, x1, y1;
     char u[10];
-    char *token;
+    char *attribute, *value, *re_string;
     char line[200];
     boolean wFlag = FALSE, hFlag = FALSE;
+#define RE_NMATCH 4
+    regmatch_t re_pmatch[RE_NMATCH];
 
-    fseek(us->f, -strlen(line), SEEK_CUR);
+    /* compile on first use */
+    if (! pre) {
+        if (regcomp(&re, svg_attr_value_re, REG_EXTENDED) != 0) {
+	    agerr(AGERR,"cannot compile regular expression %s", svg_attr_value_re);
+    	}
+	pre = &re;
+    }
+
+    fseek(us->f, 0, SEEK_SET);
     while (fgets(line, sizeof(line), us->f) != NULL && (!wFlag || !hFlag)) {
-        token = strtok(line, " ");
-        while (token != NULL && token[strlen(token)-1] != '>') {
-	    if (sscanf(token, "width=\"%lf%2s\"", &n, u) == 2) {
-	        w = svg_units_convert(n, u);
-	        wFlag = TRUE;
+	re_string = line;
+	while (regexec(&re, re_string, RE_NMATCH, re_pmatch, 0) == 0) {
+	    re_string[re_pmatch[1].rm_eo] = '\0';
+	    re_string[re_pmatch[2].rm_eo] = '\0';
+	    attribute = re_string + re_pmatch[1].rm_so;
+	    value = re_string + re_pmatch[2].rm_so;
+    	    re_string += re_pmatch[0].rm_eo + 1;
+
+	    if (strcmp(attribute,"width") == 0) {
+	        if (sscanf(value, "%lf%2s", &n, u) == 2) {
+	            w = svg_units_convert(n, u);
+	            wFlag = TRUE;
+		}
+		else if (sscanf(value, "%lf", &n) == 1) {
+	            w = svg_units_convert(n, "pt");
+	            wFlag = TRUE;
+		}
 		if (hFlag)
 		    break;
 	    }
-	    if (sscanf(token, "height=\"%lf%2s\"", &n, u) == 2) {
-	        h = svg_units_convert(n, u);
-	        hFlag = TRUE;
+	    else if (strcmp(attribute,"height") == 0) {
+	        if (sscanf(value, "%lf%2s", &n, u) == 2) {
+	            h = svg_units_convert(n, u);
+	            hFlag = TRUE;
+		}
+	        else if (sscanf(value, "%lf", &n) == 1) {
+	            h = svg_units_convert(n, "pt");
+	            hFlag = TRUE;
+		}
                 if (wFlag)
 		    break;
 	    }
-            token =  strtok(NULL, " ");
-        }
+	    else if (strcmp(attribute,"viewBox") == 0
+	      && sscanf(value, "%lf %lf %lf %lf", &x0,&y0,&x1,&y1) == 4) {
+		w = x1 - x0 + 1;
+		h = y1 - y0 + 1;
+	        wFlag = TRUE;
+	        hFlag = TRUE;
+	        break;
+	    }
+	}
     }
     us->dpi = 72;
     us->w = w;
@@ -188,6 +249,28 @@ static void png_size (usershape_t *us)
     }
 }
 
+static void webp_size (usershape_t *us)
+{
+    unsigned int w, h;
+
+    us->dpi = 0;
+    fseek(us->f, 15, SEEK_SET);
+    if (fgetc(us->f) == 'X') { //VP8X
+        fseek(us->f, 24, SEEK_SET);
+        if (get_int_lsb_first(us->f, 4, &w) && get_int_lsb_first(us->f, 4, &h)) {
+            us->w = w;
+            us->h = h;
+        }
+    }
+    else { //VP8
+        fseek(us->f, 26, SEEK_SET);
+        if (get_int_lsb_first(us->f, 2, &w) && get_int_lsb_first(us->f, 2, &h)) {
+            us->w = w;
+            us->h = h;
+        }
+    }
+}
+
 static void gif_size (usershape_t *us)
 {
     unsigned int w, h;
@@ -323,6 +406,115 @@ static void ps_size (usershape_t *us)
     }
 }
 
+#define KEY "/MediaBox"
+
+typedef struct {
+    char* s;
+    char* buf;
+    FILE* fp;
+} stream_t;
+
+static unsigned char
+nxtc (stream_t* str)
+{
+    if (fgets(str->buf, BUFSIZ, str->fp)) {
+	str->s = str->buf;
+	return *(str->s);
+    }
+    return '\0';
+	
+}
+
+#define strc(x) (*(x->s)?*(x->s):nxtc(x))
+#define stradv(x) (x->s++)
+
+static void
+skipWS (stream_t* str)
+{
+    unsigned char c;
+    while ((c = strc(str))) {
+	if (isspace(c)) stradv(str);
+	else return;
+    }
+}
+
+static int
+scanNum (char* tok, double* dp)
+{
+    char* endp;
+    double d = strtod(tok, &endp);
+
+    if (tok == endp) return 1;
+    *dp = d;
+    return 0;
+}
+
+static void
+getNum (stream_t* str, char* buf)
+{
+    int len = 0;
+    char c;
+    skipWS(str);
+    while ((c = strc(str)) && (isdigit(c) || (c == '.'))) {
+	buf[len++] = c;
+	stradv(str);
+	if (len == BUFSIZ-1) break;
+    }
+    buf[len] = '\0';
+
+    return;
+}
+
+static int
+boxof (stream_t* str, boxf* bp)
+{
+	char tok[BUFSIZ];
+
+	skipWS(str);
+	if (strc(str) != '[') return 1;
+	stradv(str);
+	getNum(str, tok);
+	if (scanNum(tok,&bp->LL.x)) return 1;
+	getNum(str, tok);
+	if (scanNum(tok,&bp->LL.y)) return 1;
+	getNum(str, tok);
+	if (scanNum(tok,&bp->UR.x)) return 1;
+	getNum(str, tok);
+	if (scanNum(tok,&bp->UR.y)) return 1;
+	return 0;
+}
+
+static int
+bboxPDF (FILE* fp, boxf* bp)
+{
+	stream_t str;
+	char* s;
+	char buf[BUFSIZ];
+	while (fgets(buf, BUFSIZ, fp)) {
+		if ((s = strstr(buf,KEY))) {
+			str.buf = buf;
+			str.s = s+(sizeof(KEY)-1);
+			str.fp = fp;
+			return boxof(&str,bp);
+		} 
+	} 
+	return 1;
+}
+
+static void pdf_size (usershape_t *us)
+{
+    boxf bb;
+
+    us->dpi = POINTS_PER_INCH;
+    fseek(us->f, 0, SEEK_SET);
+    if ( ! bboxPDF (us->f, &bb)) {
+	us->x = bb.LL.x;
+	us->y = bb.LL.y;
+        us->w = bb.UR.x - bb.LL.x;
+        us->h = bb.UR.y - bb.LL.y;
+    }
+}
+
 static void usershape_close (Dict_t * dict, Void_t * p, Dtdisc_t * disc)
 {
     usershape_t *us = (usershape_t *)p;
@@ -435,10 +627,15 @@ static usershape_t *gvusershape_open (char *name)
 	    case FT_PS:
 	        ps_size(us);
 	        break;
+	    case FT_WEBP:
+	        webp_size(us);
+	        break;
 	    case FT_SVG:
 	        svg_size(us);
 	        break;
-	    case FT_PDF:   /* no pdf_size code available */
+	    case FT_PDF:
+		pdf_size(us);
+		break;
 	    case FT_EPS:   /* no eps_size code available */
 	    default:
 	        break;
@@ -480,6 +677,8 @@ point gvusershape_size(graph_t * g, char *name)
 {
     point rv;
     pointf dpi;
+    static char* oldpath;
+    usershape_t* us;
 
     /* no shape file, no shape size */
     if (!name || (*name == '\0')) {
@@ -487,10 +686,20 @@ point gvusershape_size(graph_t * g, char *name)
 	return rv;
     }
 
+    if (!HTTPServerEnVar && (oldpath != Gvimagepath)) {
+	oldpath = Gvimagepath;
+	if (ImageDict) {
+	    dtclose(ImageDict);
+	    ImageDict = NULL;
+	}
+    }
+
     if ((dpi.y = GD_drawing(g)->dpi) >= 1.0)
 	dpi.x = dpi.y;
     else
 	dpi.x = dpi.y = (double)DEFAULT_DPI;
 
-    return gvusershape_size_dpi (gvusershape_open (name), dpi);
+    us = gvusershape_open (name);
+    rv = gvusershape_size_dpi (us, dpi);
+    return rv;
 }
