@@ -1,4 +1,4 @@
-/* $Id: gvpr.c,v 1.15 2011/01/25 16:30:49 ellson Exp $ $Revision: 1.15 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -30,6 +30,7 @@
 #include "builddate.h"
 #include "gprstate.h"
 #include "cgraph.h"
+#include "globals.h"
 #include "ingraphs.h"
 #include "compile.h"
 #include "queue.h"
@@ -44,9 +45,9 @@
 #include <getopt.h>
 #endif
 
-#define DFLT_GPRPATH    "."
-
-#define GV_USE_JUMP 4
+#ifndef DFLT_GVPRPATH
+#define DFLT_GVPRPATH    "."
+#endif
 
 static char *Info[] = {
     "gvpr",                     /* Program */
@@ -61,6 +62,7 @@ static const char *usage =
    -i         - create node induced subgraph\n\
    -a <args>  - string arguments available as ARGV[0..]\n\
    -o <ofile> - write output to <ofile>; stdout by default\n\
+   -n         - no read-ahead of input graphs\n\
    -q         - turn off warning messages\n\
    -V         - print version info\n\
    -?         - print usage info\n\
@@ -72,10 +74,12 @@ typedef struct {
     char *program;              /* program source */
     int useFile;		/* true if program comes from a file */
     int compflags;
+    int readAhead;
     char **inFiles;
     int argc;
     char **argv;
     int state;                  /* > 0 : continue; <= 0 finish */
+    int verbose;
 } options;
 
 static Sfio_t *openOut(char *name)
@@ -187,21 +191,36 @@ static int parseArgs(char *s, int argc, char ***argv)
 #define LISTSEP ':'
 #endif
 
+static Sfio_t*
+concat (char* pfx, char* sfx, char** sp)
+{
+    Sfio_t *pathp;
+    if (!(pathp = sfstropen())) {
+	error(ERROR_ERROR, "Could not open buffer");
+	return 0;
+    }
+    sfprintf(pathp, "%s%s", pfx, sfx);
+    *sp = sfstruse(pathp);
+    return pathp;
+}
+
 /* resolve:
  * Translate -f arg parameter into a pathname.
  * If arg contains '/', return arg.
- * Else search directories in GPRPATH for arg.
+ * Else search directories in GVPRPATH for arg.
  * Return NULL on error.
  * 
  * FIX - use pathinclude/pathfind
  */
-static char *resolve(char *arg)
+static char *resolve(char *arg, int Verbose)
 {
     char *path;
     char *s;
     char *cp;
+    char c;
     char *fname = 0;
     Sfio_t *fp;
+    Sfio_t *pathp = NULL;
     size_t sz;
 
 #ifdef WIN32_DLL
@@ -211,10 +230,21 @@ static char *resolve(char *arg)
 #endif
 	return strdup(arg);
 
-    path = getenv("GPRPATH");
+    path = getenv("GVPRPATH");
     if (!path)
-	path = DFLT_GPRPATH;
-
+	path = getenv("GPRPATH");  // deprecated
+    if (path && (c = *path)) {
+	if (c == LISTSEP) {
+	    pathp = concat(DFLT_GVPRPATH, path, &path); 
+	}
+	else if ((c = path[strlen(path)-1]) == LISTSEP) {
+	    pathp = concat(path, DFLT_GVPRPATH, &path); 
+	}
+    }
+    else
+	path = DFLT_GVPRPATH;
+    if (Verbose)
+	fprintf (stderr, "PATH: %s\n", path);
     if (!(fp = sfstropen())) {
 	error(ERROR_ERROR, "Could not open buffer");
 	return 0;
@@ -244,9 +274,13 @@ static char *resolve(char *arg)
     }
 
     if (!fname)
-	error(ERROR_ERROR, "Could not find file \"%s\" in GPRPATH", arg);
+	error(ERROR_ERROR, "Could not find file \"%s\" in GVPRPATH", arg);
 
     sfclose(fp);
+    if (pathp)
+	sfclose(pathp);
+    if (Verbose)
+	fprintf (stderr, "file %s resolved to %s\n", arg, fname);
     return fname;
 }
 
@@ -296,7 +330,7 @@ doFlags(char* arg, int argi, int argc, char** argv, options* opts)
 	    opts->compflags |= (SRCOUT|CLONE);
 	    break;
 	case 'f':
-	    if ((optarg = getOptarg(c, &arg, &argi, argc, argv)) && (opts->program = resolve(optarg))) {
+	    if ((optarg = getOptarg(c, &arg, &argi, argc, argv)) && (opts->program = resolve(optarg, opts->verbose))) {
 		opts->useFile = 1;
 	    }
 	    else return -1;
@@ -304,6 +338,9 @@ doFlags(char* arg, int argi, int argc, char** argv, options* opts)
 	case 'i':
 	    opts->compflags |= INDUCE;
 	    break;
+	case 'n':
+	    opts->readAhead = 0;
+	    break;
 	case 'a':
 	    if ((optarg = getOptarg(c, &arg, &argi, argc, argv))) {
 		opts->argc = parseArgs(optarg, opts->argc, &(opts->argv));
@@ -317,6 +354,9 @@ doFlags(char* arg, int argi, int argc, char** argv, options* opts)
 	case 'q':
 	    setTraceLevel (ERROR_ERROR);  /* Don't emit warning messages */
 	    break;
+	case 'v':
+	    opts->verbose = 1;
+	    break;
 	case 'V':
 	    sfprintf(sfstderr, "%s version %s (%s)\n",
 		    Info[0], Info[1], Info[2]);
@@ -364,7 +404,9 @@ static options* scanArgs(int argc, char **argv, gvpropts* uopts)
 
     opts->cmdName = argv[0];
     opts->state = 1;
+    opts->readAhead = 1;
     setErrorId (opts->cmdName);
+    opts->verbose = 0;
 
     /* estimate number of file names */
     nfiles = 0;
@@ -478,6 +520,9 @@ static Agnode_t *nextNode(Gpr_t * state, nodestream * nodes)
 
     if (state->tvroot != nodes->oldroot) {
 	np = nodes->oldroot = state->tvroot;
+    } else if (state->flags & GV_NEXT_SET) {
+	np = nodes->oldroot = state->tvroot = state->tvnext;
+	state->flags &= ~GV_NEXT_SET;
     } else if (nodes->prev) {
 	np = nodes->prev = agnxtnode(state->curgraph, nodes->prev);
     } else {
@@ -890,8 +935,8 @@ gverrorf (Expr_t *handle, Exdisc_t *discipline, int level, ...)
  *  - close non-source/non-output graphs
  *  - flag to clone target graph?
  *  - remove assignment in boolean warning if wrapped in ()
- *  - supply input edge during traversal
  *  - do automatic cast for array indices if type is known
+ *  - array initialization
  */
 int gvpr (int argc, char *argv[], gvpropts * uopts)
 {
@@ -905,6 +950,7 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
     int rv = 0;
     options* opts = 0;
     int cleanup, i, incoreGraphs;
+    Agraph_t* nextg = NULL;
 
     setErrorErrors (0);
     ingDisc.dflt = sfstdin;
@@ -919,6 +965,8 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
 	goto finish;
     }
 
+    if (opts->verbose)
+	gvstart_timer ();
     prog = parseProg(opts->program, opts->useFile);
     if (!prog) {
 	rv = 1;
@@ -968,6 +1016,8 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
     else
 	incoreGraphs = 0;
 
+    if (opts->verbose)
+	sfprintf (sfstderr, "Parse/compile/init: %.2f secs.\n", gvelapsed_sec());
     /* do begin */
     if (xprog->begin_stmt)
 	exeval(xprog->prog, xprog->begin_stmt, state);
@@ -979,8 +1029,12 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
 	else
 	    ing = newIng(0, opts->inFiles, &ingDisc);
 	
-	while ((state->curgraph = nextGraph(ing))) {
+	if (opts->verbose) gvstart_timer ();
+	for (state->curgraph = nextGraph(ing); state->curgraph; state->curgraph = nextg) {
+	    if (opts->verbose) sfprintf (sfstderr, "Read graph: %.2f secs.\n", gvelapsed_sec());
 	    state->infname = fileName(ing);
+	    if (opts->readAhead)
+		nextg = state->nextgraph = nextGraph(ing);
 	    cleanup = 0;
 
 	    for (i = 0; i < xprog->n_blocks; i++) {
@@ -991,6 +1045,7 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
 		    state->curgraph = (Agraph_t*)clone (0, (Agobj_t*)(state->curgraph));
 		state->curobj = (Agobj_t *) state->curgraph;
 		state->tvroot = 0;
+		state->tvt = TV_flat;
 		if (bp->begg_stmt)
 		    exeval(xprog->prog, bp->begg_stmt, state);
 
@@ -1004,6 +1059,7 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
 	    state->curobj = (Agobj_t *) state->curgraph;
 	    if (xprog->endg_stmt)
 		exeval(xprog->prog, xprog->endg_stmt, state);
+	    if (opts->verbose) sfprintf (sfstderr, "Finish graph: %.2f secs.\n", gvelapsed_sec());
 
 	    /* if $O == $G and $T is empty, delete $T */
 	    if ((state->outgraph == state->curgraph) &&
@@ -1026,6 +1082,11 @@ int gvpr (int argc, char *argv[], gvpropts * uopts)
 		chkClose(state->curgraph);
 	    state->target = 0;
 	    state->outgraph = 0;
+	
+	    if (opts->verbose) gvstart_timer ();
+	    if (!opts->readAhead)
+		nextg = nextGraph(ing);
+	    if (opts->verbose && nextg) sfprintf (sfstderr, "Read graph: %.2f secs.\n", gvelapsed_sec());
 	}
     }
 
