@@ -765,9 +765,9 @@ SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
     }
 
     A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J,
-					    val, type);
+					    val, type, sizeof(real));
 
-    if (D) *D = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, valD, type);
+    if (D) *D = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, valD, type, sizeof(real));
 
     free(I);
     free(J);
@@ -1060,7 +1060,12 @@ static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
 	    ap++;
 	}
 	if (ap->attrib == NULL ) {
-	    if (mapbool(s)) {
+	    int v = mapBool(s,'?');
+	    if (v == '?') {
+		agerr (AGWARN, "Unrecognized overlap value \"%s\" - using false\n", s);
+		v = FALSE;
+	    }
+	    if (v) {
 		dp->mode = adjustMode[0].mode;
 		dp->print = adjustMode[0].print;
 	    }
