@@ -1,4 +1,4 @@
-/* $Id: patchworkinit.c,v 1.9 2011/04/20 22:10:10 erg Exp $Revision: */
+/* $Id$Revision: */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -85,6 +85,9 @@ mkClusters (graph_t * g, clist_t* pclist, graph_t* parent)
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
 #endif /* WITH_CGRAPH */
         if (!strncmp(agnameof(subg), "cluster", 7)) {
+#ifdef WITH_CGRAPH
+	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+#endif
 #ifdef FDP_GEN
             GD_alg(subg) = (void *) NEW(gdata); /* freed in cleanup_subgs */
             GD_ndim(subg) = GD_ndim(parent);
@@ -113,6 +116,7 @@ static void patchwork_init_node(node_t * n)
 
 static void patchwork_init_edge(edge_t * e)
 {
+    agbindrec(e, "Agedgeinfo_t", sizeof(Agnodeinfo_t), TRUE);  // edge custom data
     /* common_init_edge(e); */
 }
 
@@ -125,11 +129,13 @@ static void patchwork_init_node_edge(graph_t * g)
 
     GD_neato_nlist(g) = N_NEW(agnnodes(g) + 1, node_t *);
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+#ifdef WITH_CGRAPH
+	agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);  // node custom data
+#endif /* WITH_CGRAPH */
 	ND_alg(n) = alg + i;
 	GD_neato_nlist(g)[i++] = n;
 	patchwork_init_node(n);
-    }
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
 	    patchwork_init_edge(e);
 	}
