@@ -1,20 +1,21 @@
+/* $Id$Revision: */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
- 
+#include "SparseMatrix.h"
 #include "spring_electrical.h"
 #include "QuadTree.h"
 #include "Multilevel.h"
@@ -29,23 +30,26 @@
 #include <string.h>
 #include <time.h>
 
-#define drand() (rand()/(real) RAND_MAX)
-
-#define DEBUG_PRINT
-#define MALLOC gmalloc
-#define REALLOC grealloc
-#define FREE free
-#define MEMCPY memcpy
-
+#define PI M_PI
 
 spring_electrical_control spring_electrical_control_new(){
   spring_electrical_control ctrl;
-  ctrl = N_GNEW(1,struct spring_electrical_control_struct);
+  ctrl = MALLOC(sizeof(struct spring_electrical_control_struct));
   ctrl->p = AUTOP;/*a negativve number default to -1. repulsive force = dist^p */
+  ctrl->q = 1;/*a positive number default to 1. Only apply to maxent. 
+		attractive force = dist^q. Stress energy = (||x_i-x_j||-d_ij)^{q+1} */
   ctrl->random_start = TRUE;/* whether to apply SE from a random layout, or from exisiting layout */
   ctrl->K = -1;/* the natural distance. If K < 0, K will be set to the average distance of an edge */
   ctrl->C = 0.2;/* another parameter. f_a(i,j) = C*dist(i,j)^2/K * d_ij, f_r(i,j) = K^(3-p)/dist(i,j)^(-p). By default C = 0.2. */
   ctrl->multilevels = FALSE;/* if <=1, single level */
+
+  //ctrl->multilevel_coarsen_scheme = COARSEN_INDEPENDENT_EDGE_SET;
+  //ctrl->multilevel_coarsen_mode = COARSEN_MODE_GENTLE;
+
+  ctrl->multilevel_coarsen_scheme = COARSEN_INDEPENDENT_EDGE_SET_HEAVEST_EDGE_PERNODE_SUPERNODES_FIRST;  /* pass on to Multilevel_control->coarsen_scheme */
+  ctrl->multilevel_coarsen_mode = COARSEN_MODE_FORCEFUL;/*alternative: COARSEN_MODE_GENTLE. pass on to Multilevel_control->coarsen_mode */
+
+
   ctrl->quadtree_size = 45;/* cut off size above which quadtree approximation is used */
   ctrl->max_qtree_level = 10;/* max level of quadtree */
   ctrl->bh = 0.6;/* Barnes-Hutt constant, if width(snode)/dist[i,snode] < bh, treat snode as a supernode.*/
@@ -59,10 +63,11 @@ spring_electrical_control spring_electrical_control_new(){
   ctrl->use_node_weights = FALSE;
   ctrl->smoothing = SMOOTHING_NONE;
   ctrl->overlap = 0;
-  ctrl->tscheme = QUAD_TREE_NORMAL;
-  ctrl->posSet = NULL;
+  ctrl->tscheme = QUAD_TREE_HYBRID;
+  ctrl->method = METHOD_SPRING_ELECTRICAL;
   ctrl->initial_scaling = -4;
-  
+  ctrl->rotation = 0.;
+  ctrl->edge_labeling_scheme = 0;
   return ctrl;
 }
 
@@ -70,29 +75,20 @@ void spring_electrical_control_delete(spring_electrical_control ctrl){
   FREE(ctrl);
 }
 
-enum {MAX_I = 20, OPT_UP = 1, OPT_DOWN = -1, OPT_INIT = 0};
-struct oned_optimizer_struct{
-  int i;
-  real work[MAX_I+1];
-  int direction;
-};
 
-typedef struct oned_optimizer_struct *oned_optimizer;
-
-
-static void oned_optimizer_delete(oned_optimizer opt){
+void oned_optimizer_delete(oned_optimizer opt){
   FREE(opt);
 }
 
-static oned_optimizer oned_optimizer_new(int i){
+oned_optimizer oned_optimizer_new(int i){
   oned_optimizer opt;
-  opt = N_GNEW(1,struct oned_optimizer_struct);
+  opt = MALLOC(sizeof(struct oned_optimizer_struct));
   opt->i = i;
   opt->direction = OPT_INIT;
   return opt;
 }
 
-static void oned_optimizer_train(oned_optimizer opt, real work){
+void oned_optimizer_train(oned_optimizer opt, real work){
   int i = opt->i;
  
   assert(i >= 0);
@@ -130,7 +126,7 @@ static void oned_optimizer_train(oned_optimizer opt, real work){
   }
 }
 
-static int oned_optimizer_get(oned_optimizer opt){
+int oned_optimizer_get(oned_optimizer opt){
   return opt->i;
 }
 
@@ -153,49 +149,6 @@ real average_edge_length(SparseMatrix A, int dim, real *coord){
   return dist/ia[A->m];
 }
 
-static real
-initPos (SparseMatrix A, int n, int dim, real* x, spring_electrical_control ctrl)
-{
-  int i, j;
-  char* posSet;
-
-  if (ctrl->random_start){
-    srand(ctrl->random_seed);
-    if ((posSet = ctrl->posSet)) {
-	for (i = 0; i < n; i++) {
-	    if (posSet[i]) continue;
-	    for (j = 0; j < dim; j++) {
-		x[dim*i + j] = drand();
-	    }
-	}
-    }
-    else
-	for (i = 0; i < dim*n; i++) x[i] = drand();
-  }
-
-  if (ctrl->K < 0){
-    ctrl->K = average_edge_length(A, dim, x);
-  }
-
-  return ctrl->K;
-}
-
-real distance_cropped(real *x, int dim, int i, int j){
-  int k;
-  real dist = 0.;
-  for (k = 0; k < dim; k++) dist += (x[i*dim+k] - x[j*dim + k])*(x[i*dim+k] - x[j*dim + k]);
-  dist = sqrt(dist);
-  return MAX(dist, MINDIST);
-}
-
-real distance(real *x, int dim, int i, int j){
-  int k;
-  real dist = 0.;
-  for (k = 0; k < dim; k++) dist += (x[i*dim+k] - x[j*dim + k])*(x[i*dim+k] - x[j*dim + k]);
-  dist = sqrt(dist);
-  return dist;
-}
-
 #ifdef ENERGY
 static real spring_electrical_energy(int dim, SparseMatrix A, real *x, real p, real CRK, real KP){
       /* 1. Grad[||x-y||^k,x] = k||x-y||^(k-1)*0.5*(x-y)/||x-y|| = k/2*||x-y||^(k-2) (x-y) 
@@ -237,6 +190,20 @@ static real spring_electrical_energy(int dim, SparseMatrix A, real *x, real p, r
 void export_embedding(FILE *fp, int dim, SparseMatrix A, real *x, real *width){
   int i, j, k, *ia=A->ia, *ja = A->ja;
   int ne = 0;
+  real xsize, ysize, xmin, xmax, ymin, ymax;
+
+  xmax = xmin = x[0];
+  ymax = ymin = x[1];
+  for (i = 0; i < A->m; i++){
+    xmax = MAX(xmax, x[i*dim]);
+    xmin = MIN(xmin, x[i*dim]);
+    ymax = MAX(ymax, x[i*dim+1]);
+    ymin = MIN(ymin, x[i*dim+1]);
+  }
+  xsize = xmax-xmin;
+  ysize = ymax-ymin;
+  xsize = MAX(xsize, ysize);
+
   if (dim == 2){
     fprintf(fp,"Graphics[{GrayLevel[0.5],Line[{");
   } else {
@@ -261,11 +228,19 @@ void export_embedding(FILE *fp, int dim, SparseMatrix A, real *x, real *width){
     }
   }
 
-  fprintf(fp,"}],Hue[%f],",/*drand()*/1.);
+  fprintf(fp,"}],Hue[%f]",/*drand()*/1.);
  
+  if (width && dim == 2){
+    for (i = 0; i < A->m; i++){
+      if (i >= 0) fprintf(fp,",");
+      fprintf(fp,"(*%f,%f*){GrayLevel[.5,.5],Rectangle[{%f,%f},{%f,%f}]}", width[i*dim], width[i*dim+1],x[i*dim] - width[i*dim] + 5, x[i*dim+1] - width[i*dim+1] + 5,
+	      x[i*dim] + width[i*dim] - 5, x[i*dim+1] + width[i*dim+1] - 5);
+    }
+  }
+
   if (A->m < 100){
     for (i = 0; i < A->m; i++){
-      if (i > 0) fprintf(fp,",");
+      if (i >= 0) fprintf(fp,",");
       fprintf(fp,"Text[%d,{",i+1);
       for (k = 0; k < dim; k++) {
 	if (k > 0) fprintf(fp,",");
@@ -274,7 +249,7 @@ void export_embedding(FILE *fp, int dim, SparseMatrix A, real *x, real *width){
       fprintf(fp,"}]");
     }
   } else if (A->m < 500000){
-    fprintf(fp, "Point[{");
+    fprintf(fp, ", Point[{");
     for (i = 0; i < A->m; i++){
       if (i > 0) fprintf(fp,",");
       fprintf(fp,"{");
@@ -289,16 +264,8 @@ void export_embedding(FILE *fp, int dim, SparseMatrix A, real *x, real *width){
       fprintf(fp,"{}");
   }
 
-  if (width && dim == 2){
-    fprintf(fp, ",");
-    for (i = 0; i < A->m; i++){
-      if (i > 0) fprintf(fp,",");
-      fprintf(fp,"(*%f,%f*){GrayLevel[.5,.5],Rectangle[{%f,%f},{%f,%f}]}", width[i*dim], width[i*dim+1],x[i*dim] - width[i*dim], x[i*dim+1] - width[i*dim+1],
-	      x[i*dim] + width[i*dim], x[i*dim+1] + width[i*dim+1]);
-    }
-  }
 
- fprintf(fp,"},ImageSize->600]\n");
+ fprintf(fp,"},ImageSize->%f]\n", 2*xsize/2);
 
 }
 
@@ -323,14 +290,14 @@ static real update_step(int adaptive_cooling, real step, real Fnorm, real Fnorm0
 void check_real_array_size(real **a, int len, int *lenmax){
   if (len >= *lenmax){
     *lenmax = len + MAX((int) 0.2*len, 10);
-    *a = RALLOC((*lenmax),*a,real);
+    *a = REALLOC(*a, sizeof(real)*(*lenmax));
   }
   
 }
 void check_int_array_size(int **a, int len, int *lenmax){
   if (len >= *lenmax){
     *lenmax = len + MAX((int) 0.2*len, 10);
-    *a = RALLOC((*lenmax),*a,int);
+    *a = REALLOC(*a, sizeof(int)*(*lenmax));
   }
   
 }
@@ -344,14 +311,14 @@ real get_angle(real *x, int dim, int i, int j){
     y[k] = x[j*dim+k] - x[i*dim+k];
   }
   if (ABS(y[0]) <= ABS(y[1])*eps){
-    if (y[1] > 0) return 0.5*M_PI;
-    return 1.5*M_PI;
+    if (y[1] > 0) return 0.5*PI;
+    return 1.5*PI;
   }
   res = atan(y[1]/y[0]);
   if (y[0] > 0){
-    if (y[1] < 0) res = 2*M_PI+res;
+    if (y[1] < 0) res = 2*PI+res;
   } else if (y[0] < 0){
-    res = res + M_PI;
+    res = res + PI;
   }
   return res;
 }
@@ -387,9 +354,9 @@ static void beautify_leaves(int dim, SparseMatrix A, real *x){
 
   assert(!SparseMatrix_has_diagonal(A));
 
-  checked = N_GNEW(m,int);
-  angles = N_GNEW(nangles_max,real);
-  leaves = N_GNEW(nleaves_max,int);
+  checked = MALLOC(sizeof(int)*m);
+  angles = MALLOC(sizeof(real)*nangles_max);
+  leaves = MALLOC(sizeof(int)*nleaves_max);
 
 
   for (i = 0; i < m; i++) checked[i] = FALSE;
@@ -424,17 +391,18 @@ static void beautify_leaves(int dim, SparseMatrix A, real *x){
 	    ang1 = angles[k]; ang2 = angles[k+1];
 	  }
 	}
-	if (2*M_PI + angles[0] - angles[nangles - 1] > maxang){
-	  maxang = 2*M_PI + angles[0] - angles[nangles - 1];
+	if (2*PI + angles[0] - angles[nangles - 1] > maxang){
+	  maxang = 2*PI + angles[0] - angles[nangles - 1];
 	  ang1 = angles[nangles - 1];
-	  ang2 = 2*M_PI + angles[0];
+	  ang2 = 2*PI + angles[0];
 	}
       } else {
-	ang1 = 0; ang2 = 2*M_PI; maxang = 2*M_PI;
+	ang1 = 0; ang2 = 2*PI; maxang = 2*PI;
       }
-      pad = MAX(maxang - M_PI*0.166667*(nleaves-1), 0)*0.5;
+      pad = MAX(maxang - PI*0.166667*(nleaves-1), 0)*0.5;
       ang1 += pad*0.95;
       ang2 -= pad*0.95;
+ang1 = 0; ang2 = 2*PI; maxang = 2*PI;
       assert(ang2 >= ang1);
       step = 0.;
       if (nleaves > 1) step = (ang2 - ang1)/(nleaves - 1);
@@ -527,15 +495,20 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
   ia = A->ia;
   ja = A->ja;
 
-  K = initPos (A, n, dim, x, ctrl);
-
+  if (ctrl->random_start){
+    srand(ctrl->random_seed);
+    for (i = 0; i < dim*n; i++) x[i] = drand();
+  }
+  if (K < 0){
+    ctrl->K = K = average_edge_length(A, dim, x);
+  }
   if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
   CRK = pow(C, (2.-p)/3.)/K;
 
-  xold = N_GNEW(dim*n,real); 
-  force = N_GNEW(dim*n,real);
+  xold = MALLOC(sizeof(real)*dim*n); 
+  force = MALLOC(sizeof(real)*dim*n);
 
   do {
 #ifdef TIME
@@ -567,7 +540,7 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
     start = clock();
 #endif
 
-    QuadTree_get_repulvice_force(qt, force, x, ctrl->bh, p, KP, counts, flag);
+    QuadTree_get_repulsive_force(qt, force, x, ctrl->bh, p, KP, counts, flag);
 
     assert(!(*flag));
 
@@ -615,11 +588,12 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
 #ifdef TIME
       qtree_cpu0 = qtree_cpu - qtree_cpu0;
       qtree_new_cpu0 = qtree_new_cpu - qtree_new_cpu0;
-      if (Verbose) fprintf(stderr, "\r iter=%d cpu=%.2f, quadtree=%.2f quad_force=%.2f other=%.2f counts={%.2f,%.2f,%.2f} step=%f Fnorm=%f nz=%d  K=%f qtree_lev = %d",
+      /*      if (Verbose) fprintf(stderr, "\r iter=%d cpu=%.2f, quadtree=%.2f quad_force=%.2f other=%.2f counts={%.2f,%.2f,%.2f} step=%f Fnorm=%f nz=%d  K=%f qtree_lev = %d",
 			   iter, ((real) (clock() - start2)) / CLOCKS_PER_SEC, qtree_new_cpu0, 
 			   qtree_cpu0,((real) (clock() - start2))/CLOCKS_PER_SEC - qtree_cpu0 - qtree_new_cpu0, 
 			   counts[0], counts[1], counts[2],
 			   step, Fnorm, A->nz,K,max_qtree_level);
+      */
       qtree_cpu0 = qtree_cpu;
       qtree_new_cpu0 = qtree_new_cpu;
 #endif
@@ -699,14 +673,14 @@ void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_electrica
 
   m = A->m, n = A->n;
   if (n <= 0 || dim <= 0) return;
-  force = N_GNEW(n*dim,real);
+  force = MALLOC(sizeof(real)*n*dim);
 
   if (n >= ctrl->quadtree_size) {
     USE_QT = TRUE;
     qtree_level_optimizer = oned_optimizer_new(max_qtree_level);
-    center = N_GNEW(nsupermax*dim,real);
-    supernode_wgts = N_GNEW(nsupermax,real);
-    distances = N_GNEW(nsupermax,real);
+    center = MALLOC(sizeof(real)*nsupermax*dim);
+    supernode_wgts = MALLOC(sizeof(real)*nsupermax);
+    distances = MALLOC(sizeof(real)*nsupermax);
   }
   USE_QT = FALSE;
   *flag = 0;
@@ -719,8 +693,13 @@ void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_electrica
   ia = A->ia;
   ja = A->ja;
 
-  K = initPos (A, n, dim, x, ctrl);
-
+  if (ctrl->random_start){
+    srand(ctrl->random_seed);
+    for (i = 0; i < dim*n; i++) x[i] = drand();
+  }
+  if (K < 0){
+    ctrl->K = K = average_edge_length(A, dim, x);
+  }
   if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
@@ -738,8 +717,8 @@ void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_electrica
   }
 #endif
 
-  f = N_GNEW(dim,real);
-  xold = N_GNEW(dim*n,real); 
+  f = MALLOC(sizeof(real)*dim);
+  xold = MALLOC(sizeof(real)*dim*n); 
   do {
     for (i = 0; i < dim*n; i++) force[i] = 0;
 
@@ -957,9 +936,9 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
   if (n >= ctrl->quadtree_size) {
     USE_QT = TRUE;
     qtree_level_optimizer = oned_optimizer_new(max_qtree_level);
-    center = N_GNEW(nsupermax*dim,real);
-    supernode_wgts = N_GNEW(nsupermax,real);
-    distances = N_GNEW(nsupermax,real);
+    center = MALLOC(sizeof(real)*nsupermax*dim);
+    supernode_wgts = MALLOC(sizeof(real)*nsupermax);
+    distances = MALLOC(sizeof(real)*nsupermax);
   }
   *flag = 0;
   if (m != n) {
@@ -971,8 +950,13 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
   ia = A->ia;
   ja = A->ja;
 
-  K = initPos (A, n, dim, x, ctrl);
-
+  if (ctrl->random_start){
+    srand(ctrl->random_seed);
+    for (i = 0; i < dim*n; i++) x[i] = drand();
+  }
+  if (K < 0){
+    ctrl->K = K = average_edge_length(A, dim, x);
+  }
   if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
@@ -990,8 +974,8 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
   }
 #endif
 
-  f = N_GNEW(dim,real);
-  xold = N_GNEW(dim*n,real); 
+  f = MALLOC(sizeof(real)*dim);
+  xold = MALLOC(sizeof(real)*dim*n); 
   do {
     iter++;
     xold = MEMCPY(xold, x, sizeof(real)*dim*n);
@@ -1163,6 +1147,320 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
 
 }
 
+static void scale_coord(int n, int dim, real *x, int *id, int *jd, real *d, real dj){
+  int i, j, k;
+  real w_ij, dist, s = 0, stop = 0, sbot = 0., nz = 0;
+
+  if (dj == 0.) return;
+  for (i = 0; i < n; i++){
+    for (j = id[i]; j < id[i+1]; j++){
+      if (jd[j] == i) continue;
+      dist = distance_cropped(x, dim, i, jd[j]);
+      if (d){
+	dj = d[j];
+      } 
+      assert(dj > 0);
+      w_ij = 1./(dj*dj);
+      /* spring force */
+      for (k = 0; k < dim; k++){
+	stop += w_ij*dj*dist;
+	sbot += w_ij*dist*dist;
+      }
+      s += dist; nz++;
+    }
+  }
+  s = stop/sbot;
+  for (i = 0; i < n*dim; i++) x[i] *= s;
+  fprintf(stderr,"scaling factor = %f\n",s);
+}
+
+static real dmean_get(int n, int *id, int *jd, real* d){
+  real dmean = 0;
+  int i, j;
+
+  if (!d) return 1.;
+  for (i = 0; i < n; i++){
+    for (j = id[i]; j < id[i+1]; j++){
+      dmean += d[j];
+    }
+  }
+  return dmean/((real) id[n]);
+}
+
+void spring_maxent_embedding(int dim, SparseMatrix A0, SparseMatrix D, spring_electrical_control ctrl, real *node_weights, real *x, real rho, int *flag){
+  /* x is a point to a 1D array, x[i*dim+j] gives the coordinate of the i-th node at dimension j. 
+
+     Minimize \Sum_{(i,j)\in E} w_ij (||x_i-x_j||-d_ij)^2 - \rho \Sum_{(i,j)\NotIn E} Log ||x_i-x_j||
+
+     or
+
+     Minimize \Sum_{(i,j)\in E} w_ij (||x_i-x_j||-d_ij)^2 - \rho \Sum_{(i,j)\NotIn E} ||x_i-x_j||^p
+
+     The derivatives are
+
+     d E/d x_i = \Sum_{(i,j)\in E} w_ij (||x_i-x_j||-d_ij) (x_i-x_j)/||x_i-x_j|| - \rho \Sum_{(i,j)\NotIn E} (x_i-x_j)/||x_i-x_j||^2
+
+     or
+
+      d E/d x_i = \Sum_{(i,j)\in E} w_ij (||x_i-x_j||-d_ij) (x_i-x_j)/||x_i-x_j|| - \rho \Sum_{(i,j)\NotIn E} ||x_i-x_j||^(p-2) (x_i-x_j)
+    
+      if D == NULL, unit weight assumed
+
+  */
+  SparseMatrix A = A0;
+  int m, n;
+  int i, j, k;
+  real p = ctrl->p, C = ctrl->C, tol = ctrl->tol, maxiter = ctrl->maxiter, cool = ctrl->cool, step = ctrl->step, w_ij, dj = 1.;
+  int *ia = NULL, *ja = NULL;
+  int *id = NULL, *jd = NULL;
+  real *d, dmean;
+  real *xold = NULL;
+  real *f = NULL, dist, F, Fnorm = 0, Fnorm0;
+  int iter = 0;
+  int adaptive_cooling = ctrl->adaptive_cooling;
+  QuadTree qt = NULL;
+  int USE_QT = FALSE;
+  int nsuper = 0, nsupermax = 10;
+  real *center = NULL, *supernode_wgts = NULL, *distances = NULL, nsuper_avg, counts = 0;
+  int max_qtree_level = 10;
+#ifdef DEBUG
+  double stress = 0;
+#endif
+
+  if (!A) return;
+  m = A->m, n = A->n;
+  if (n <= 0 || dim <= 0) return;
+
+  if (ctrl->tscheme != QUAD_TREE_NONE && n >= ctrl->quadtree_size) {
+    USE_QT = TRUE;
+    center = MALLOC(sizeof(real)*nsupermax*dim);
+    supernode_wgts = MALLOC(sizeof(real)*nsupermax);
+    distances = MALLOC(sizeof(real)*nsupermax);
+  }
+
+  *flag = 0;
+  if (m != n) {
+    *flag = ERROR_NOT_SQUARE_MATRIX;
+    goto RETURN;
+  }
+
+
+  assert(A->format == FORMAT_CSR);
+  A = SparseMatrix_symmetrize(A, TRUE);
+  ia = A->ia;
+  ja = A->ja;
+  if (D){
+    id = D->ia;
+    jd = D->ja;
+    d = (real*) D->a;
+  } else {
+    id = ia; jd = ja; d = NULL;
+  }
+  if (rho < 0) {
+    dmean = dmean_get(n, id, jd, d);
+    rho = rho*(id[n]/((((real) n)*((real) n)) - id[n]))/pow(dmean, p+1);
+    fprintf(stderr,"dmean = %f, rho = %f\n",dmean, rho);
+  }
+
+  if (ctrl->random_start){
+    fprintf(stderr, "send random coordinates\n");
+    srand(ctrl->random_seed);
+    for (i = 0; i < dim*n; i++) x[i] = drand();
+  /* rescale x to give minimum stress:
+     Min \Sum_{(i,j)\in E} w_ij (s ||x_i-x_j||-d_ij)^2
+     thus
+     s = (\Sum_{(ij)\in E} w_ij d_ij ||x_i-x_j||)/(\Sum_{(i,j)\in E} w_ij ||x_i-x_j||^2)
+  */
+
+  } 
+  scale_coord(n, dim, x, id, jd, d, dj);
+
+
+
+  if (C < 0) ctrl->C = C = 0.2;
+  if (p >= 0) ctrl->p = p = -1;
+
+#ifdef DEBUG_0
+  {
+    FILE *f;
+    char fname[10000];
+    strcpy(fname,"/tmp/graph_layout_0_");
+    sprintf(&(fname[strlen(fname)]), "%d",n);
+    f = fopen(fname,"w");
+    export_embedding(f, dim, A, x, NULL);
+    fclose(f);
+  }
+#endif
+
+  f = MALLOC(sizeof(real)*dim);
+  xold = MALLOC(sizeof(real)*dim*n); 
+  do {
+    iter++;
+    xold = MEMCPY(xold, x, sizeof(real)*dim*n);
+    Fnorm0 = Fnorm;
+    Fnorm = 0.;
+    nsuper_avg =- 0;
+#ifdef DEBUG
+    stress = 0;
+#endif
+
+    if (USE_QT) {
+      if (ctrl->use_node_weights){
+	qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x, node_weights);
+      } else {
+	qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x, NULL);
+      }
+    }
+
+    /*
+      .    d E/d x_i = \Sum_{(i,j)\in E} w_ij (||x_i-x_j||-d_ij) (x_i-x_j)/||x_i-x_j|| - \rho \Sum_{(i,j)\NotIn E} (x_i-x_j)/||x_i-x_j||^2
+      or
+      .    d E/d x_i = \Sum_{(i,j)\in E} w_ij (||x_i-x_j||-d_ij) (x_i-x_j)/||x_i-x_j|| - \rho \Sum_{(i,j)\NotIn E} ||x_i-x_j||^(p-2) (x_i-x_j)
+    */
+    for (i = 0; i < n; i++){
+      for (k = 0; k < dim; k++) f[k] = 0.;
+
+      /* spring (attractive or repulsive) force   w_ij (||x_i-x_j||-d_ij) (x_i-x_j)/||x_i-x_j|| */
+      for (j = id[i]; j < id[i+1]; j++){
+	if (jd[j] == i) continue;
+	dist = distance_cropped(x, dim, i, jd[j]);
+	if (d){
+	  dj = d[j];
+	} 
+	assert(dj > 0);
+	/* spring force */
+	if (ctrl->q == 2){
+	  w_ij = 1./(dj*dj*dj);
+	  for (k = 0; k < dim; k++){
+	    f[k] += -w_ij*(x[i*dim+k] - x[jd[j]*dim+k])*(dist - dj)*(dist - dj)/dist;
+	  }
+	} else if (ctrl->q == 1){/* square stress force */
+	  w_ij = 1./(dj*dj);
+	  for (k = 0; k < dim; k++){
+	    f[k] += -w_ij*(x[i*dim+k] - x[jd[j]*dim+k])*(dist - dj)/dist;
+	  }
+	} else {
+	  w_ij = 1./pow(dj, ctrl->q + 1);
+	  for (k = 0; k < dim; k++){
+	    f[k] += -w_ij*(x[i*dim+k] - x[jd[j]*dim+k])*pow(dist - dj, ctrl->q)/dist;
+	  }
+	}
+
+#ifdef DEBUG
+	w_ij = 1./(dj*dj);
+	for (k = 0; k < dim; k++){
+	  stress += (dist - dj)*(dist - dj)*w_ij;
+	}
+#endif
+
+
+	/* discount repulsive force between neighboring vertices which will be applied next, that way there is no
+	   repulsive forces between neighboring vertices */
+	if (ctrl->use_node_weights && node_weights){
+	  for (k = 0; k < dim; k++){
+	    if (p == -1){
+	      f[k] -= rho*node_weights[j]*(x[i*dim+k] - x[jd[j]*dim+k])/(dist*dist);
+	    } else {
+	      f[k] -= rho*node_weights[j]*(x[i*dim+k] - x[jd[j]*dim+k])/pow(dist, 1.- p);
+	    }
+	  }
+	} else {
+	  for (k = 0; k < dim; k++){
+	    if (p == -1){
+	      f[k] -= rho*(x[i*dim+k] - x[jd[j]*dim+k])/(dist*dist);
+	    } else {
+	      f[k] -= rho*(x[i*dim+k] - x[jd[j]*dim+k])/pow(dist, 1.- p);
+	    }
+	  }
+
+	}
+
+      }
+
+      /* repulsive force ||x_i-x_j||^(1 - p) (x_i - x_j) */
+      if (USE_QT){
+	QuadTree_get_supernodes(qt, ctrl->bh, &(x[dim*i]), i, &nsuper, &nsupermax, 
+				&center, &supernode_wgts, &distances, &counts, flag);
+	nsuper_avg += nsuper;
+	if (*flag) goto RETURN;
+	for (j = 0; j < nsuper; j++){
+	  dist = MAX(distances[j], MINDIST);
+	  for (k = 0; k < dim; k++){
+	    if (p == -1){
+	      f[k] += rho*supernode_wgts[j]*(x[i*dim+k] - center[j*dim+k])/(dist*dist);
+	    } else {
+	      f[k] += rho*supernode_wgts[j]*(x[i*dim+k] - center[j*dim+k])/pow(dist, 1.- p);
+	    }
+	  }
+	}
+      } else {
+	if (ctrl->use_node_weights && node_weights){
+	  for (j = 0; j < n; j++){
+	    if (j == i) continue;
+	    dist = distance_cropped(x, dim, i, j);
+	    for (k = 0; k < dim; k++){
+	      if (p == -1){
+		f[k] += rho*node_weights[j]*(x[i*dim+k] - x[j*dim+k])/(dist*dist);
+	      } else {
+		f[k] += rho*node_weights[j]*(x[i*dim+k] - x[j*dim+k])/pow(dist, 1.- p);
+	      }
+	    }
+	  }
+	} else {
+	  for (j = 0; j < n; j++){
+	    if (j == i) continue;
+	    dist = distance_cropped(x, dim, i, j);
+	    for (k = 0; k < dim; k++){
+	      if (p == -1){
+		f[k] += rho*(x[i*dim+k] - x[j*dim+k])/(dist*dist);
+	      } else {
+		f[k] += rho*(x[i*dim+k] - x[j*dim+k])/pow(dist, 1.- p);
+	      }
+	    }
+	  }
+	}
+      }
+	
+      /* normalize force */
+      F = 0.;
+      for (k = 0; k < dim; k++) F += f[k]*f[k];
+      F = sqrt(F);
+      Fnorm += F;
+
+      if (F > 0) for (k = 0; k < dim; k++) f[k] /= F;
+
+      for (k = 0; k < dim; k++) x[i*dim+k] += step*f[k];
+
+    }/* done vertex i */
+
+    if (qt) QuadTree_delete(qt);
+    nsuper_avg /= n;
+#ifdef DEBUG_PRINT
+    stress /= (double) A->nz;
+    if (Verbose) {
+      fprintf(stderr, "\r                iter = %d, step = %f Fnorm = %f nsuper = %d nz = %d stress = %f                                  ",iter, step, Fnorm, (int) nsuper_avg,A->nz, stress);
+    }
+#endif
+    
+    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0, cool);
+  } while (step > tol && iter < maxiter);
+
+#ifdef DEBUG_PRINT
+  if (Verbose) fputs("\n", stderr);
+#endif
+
+
+  if (ctrl->beautify_leaves) beautify_leaves(dim, A, x);
+
+ RETURN:
+  if (xold) FREE(xold);
+  if (A != A0) SparseMatrix_delete(A);
+  if (f) FREE(f);
+  if (center) FREE(center);
+  if (supernode_wgts) FREE(supernode_wgts);
+  if (distances) FREE(distances);
+
+}
 
 
 
@@ -1193,9 +1491,9 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
 
   if (n >= ctrl->quadtree_size) {
     USE_QT = TRUE;
-    center = N_GNEW(nsupermax*dim,real);
-    supernode_wgts = N_GNEW(nsupermax,real);
-    distances = N_GNEW(nsupermax,real);
+    center = MALLOC(sizeof(real)*nsupermax*dim);
+    supernode_wgts = MALLOC(sizeof(real)*nsupermax);
+    distances = MALLOC(sizeof(real)*nsupermax);
   }
   *flag = 0;
   if (m != n) {
@@ -1210,8 +1508,13 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
   jd = D->ja;
   d = (real*) D->a;
 
-  K = initPos (A, n, dim, x, ctrl);
-
+  if (ctrl->random_start){
+    srand(ctrl->random_seed);
+    for (i = 0; i < dim*n; i++) x[i] = drand();
+  }
+  if (K < 0){
+    ctrl->K = K = average_edge_length(A, dim, x);
+  }
   if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
@@ -1229,8 +1532,8 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
   }
 #endif
 
-  f = N_GNEW(dim,real);
-  xold = N_GNEW(dim*n,real); 
+  f = MALLOC(sizeof(real)*dim);
+  xold = MALLOC(sizeof(real)*dim*n); 
   do {
     iter++;
     xold = MEMCPY(xold, x, sizeof(real)*dim*n);
@@ -1392,7 +1695,7 @@ static void interpolate2(int dim, SparseMatrix A, real *x){
   int i, j, k, *ia = A->ia, *ja = A->ja, nz, m = A->m;
   real alpha = 0.5, beta, *y;
 
-  y = N_GNEW(dim*m,real);
+  y = MALLOC(sizeof(real)*dim*m);
   for (k = 0; k < dim*m; k++) y[k] = 0;
   for (i = 0; i < m; i++){
     nz = 0;
@@ -1415,11 +1718,11 @@ static void interpolate2(int dim, SparseMatrix A, real *x){
 
 */
 
-static void interpolate0(int dim, SparseMatrix A, real *x){
+void interpolate_coord(int dim, SparseMatrix A, real *x){
   int i, j, k, *ia = A->ia, *ja = A->ja, nz;
   real alpha = 0.5, beta, *y;
 
-  y = N_GNEW(dim,real);
+  y = MALLOC(sizeof(real)*dim);
   for (i = 0; i < A->m; i++){
     for (k = 0; k < dim; k++) y[k] = 0;
     nz = 0;
@@ -1444,7 +1747,7 @@ static void prolongate(int dim, SparseMatrix A, SparseMatrix P, SparseMatrix R,
 
   /* xu yao rao dong */
   if (coarsen_scheme_used > EDGE_BASED_STA && coarsen_scheme_used < EDGE_BASED_STO){
-    interpolate0(dim, A, y);
+    interpolate_coord(dim, A, y);
     nc = R->m;
     ia = R->ia;
     ja = R->ja;
@@ -1464,7 +1767,7 @@ int power_law_graph(SparseMatrix A){
   int *mask, m, max = 0, i, *ia = A->ia, *ja = A->ja, j, deg;
   int res = FALSE;
   m = A->m;
-  mask = N_GNEW((m+1),int);
+  mask = MALLOC(sizeof(int)*(m+1));
 
   for (i = 0; i < m + 1; i++){
     mask[i] = 0;
@@ -1535,21 +1838,169 @@ void pcp_rotate(int n, int dim, real *x){
 
 
 }
+ 
+static void rotate(int n, int dim, real *x, real angle){
+  int i, k;
+  real axis[2], center[2], x0, x1;
+  real radian = 3.14159/180;
+
+  assert(dim == 2);
+  for (i = 0; i < dim; i++) center[i] = 0;
+  for (i = 0; i < n; i++){
+    for (k = 0; k < dim; k++){
+      center[k] += x[i*dim+k];
+    }
+  }
+  for (i = 0; i < dim; i++) center[i] /= n;
+  for (i = 0; i < n; i++){
+    for (k = 0; k < dim; k++){
+      x[dim*i+k] =  x[dim*i+k] - center[k];
+    }
+  }
+  axis[0] = cos(-angle*radian);
+  axis[1] = sin(-angle*radian);
+  for (i = 0; i < n; i++){
+    x0 = x[dim*i]*axis[0]+x[dim*i+1]*axis[1];
+    x1 = -x[dim*i]*axis[1]+x[dim*i+1]*axis[0];
+    x[dim*i] = x0;
+    x[dim*i + 1] = x1;
+  }
+
+  
+}
+
+static void attach_edge_label_coordinates(int dim, SparseMatrix A, int n_edge_label_nodes, int *edge_label_nodes, real *x, real *x2){
+  int *mask;
+  int i, ii, j, k;
+  int nnodes = 0;
+  real len;
+
+  mask = MALLOC(sizeof(int)*A->m);
+
+  for (i = 0; i < A->m; i++) mask[i] = 1;
+  for (i = 0; i < n_edge_label_nodes; i++) {
+    if (edge_label_nodes[i] >= 0 && edge_label_nodes[i] < A->m) mask[edge_label_nodes[i]] = -1;
+  }
+
+  for (i = 0; i < A->m; i++) {
+    if (mask[i] >= 0) mask[i] = nnodes++;
+  }
 
 
-void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_control ctrl, real *node_weights, real *label_sizes, 
-					    real *x, int *flag){
+  for (i = 0; i < A->m; i++){
+    if (mask[i] >= 0){
+      for (k = 0; k < dim; k++) x[i*dim+k] = x2[mask[i]*dim+k];
+    }
+  }
+
+  for (i = 0; i < n_edge_label_nodes; i++){
+    ii = edge_label_nodes[i];
+    len = A->ia[ii+1] - A->ia[ii];
+    assert(len >= 2); /* should just be 2 */
+    assert(mask[ii] < 0);
+    for (k = 0; k < dim; k++) {
+      x[ii*dim+k] = 0;
+    }
+    for (j = A->ia[ii]; j < A->ia[ii+1]; j++){
+      for (k = 0; k < dim; k++){
+	x[ii*dim+k] += x[(A->ja[j])*dim+k];
+      }
+    }
+    for (k = 0; k < dim; k++) {
+      x[ii*dim+k] /= len;
+    }
+  }
+
+  FREE(mask);
+}
+
+static SparseMatrix shorting_edge_label_nodes(SparseMatrix A, int n_edge_label_nodes, int *edge_label_nodes){
+  int *mask;
+  int i, id = 0, nz, j, jj, ii;
+  int *ia = A->ia, *ja = A->ja, *irn = NULL, *jcn = NULL;
+  SparseMatrix B;
+
+  mask = MALLOC(sizeof(int)*A->m);
+
+  for (i = 0; i < A->m; i++) mask[i] = 1;
+
+  for (i = 0; i < n_edge_label_nodes; i++){
+    mask[edge_label_nodes[i]] = -1;
+  }
+
+  for (i = 0; i < A->m; i++) {
+    if (mask[i] > 0) mask[i] = id++;
+  }
+
+  nz  = 0;
+  for (i = 0; i < A->m; i++){
+    if (mask[i] < 0) continue;
+    for (j = ia[i]; j < ia[i+1]; j++){
+      if (mask[ja[j]] >= 0) {
+	nz++;
+	continue;
+      }
+      ii = ja[j];
+      for (jj = ia[ii]; jj < ia[ii+1]; jj++){
+	if (ja[jj] != i && mask[ja[jj]] >= 0) nz++;
+      }
+    }
+  }
+
+  if (nz > 0) {
+    irn = MALLOC(sizeof(int)*nz);
+    jcn = MALLOC(sizeof(int)*nz);
+  }
+
+  nz = 0;
+  for (i = 0; i < A->m; i++){
+    if (mask[i] < 0) continue;
+    for (j = ia[i]; j < ia[i+1]; j++){
+      if (mask[ja[j]] >= 0) {
+	irn[nz] = mask[i];
+	jcn[nz++] = mask[ja[j]];
+	if (mask[i] == 68 || mask[ja[j]] == 68){
+	  fprintf(stderr, "xxx %d %d\n",mask[i], mask[ja[j]]);
+	  mask[i] = mask[i];
+	}
+	continue;
+      }
+      ii = ja[j];
+      for (jj = ia[ii]; jj < ia[ii+1]; jj++){
+	if (ja[jj] != i && mask[ja[jj]] >= 0) {
+	    irn[nz] = mask[i];
+	    jcn[nz++] = mask[ja[jj]];
+	    if (mask[i] == 68 || mask[ja[jj]] == 68){
+	      fprintf(stderr, "%d %d\n",mask[i], mask[ja[jj]]);
+	      mask[i] = mask[i];
+	    }
+	}
+      }
+    }
+  }
+
+  B = SparseMatrix_from_coordinate_arrays(nz, id, id, irn, jcn, NULL, MATRIX_TYPE_PATTERN, sizeof(real));
+
+  FREE(irn);
+  FREE(jcn);
+  FREE(mask);
+  return B;
+
+}
+
+static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0, SparseMatrix D0, spring_electrical_control ctrl, real *node_weights, real *label_sizes, 
+					    real *x, int n_edge_label_nodes, int *edge_label_nodes, int *flag){
   
 
   Multilevel_control mctrl = NULL;
   int n, plg, coarsen_scheme_used;
-  SparseMatrix A = A0, P = NULL;
+  SparseMatrix A = A0, D = D0, P = NULL;
   Multilevel grid, grid0;
   real *xc = NULL, *xf = NULL;
+  struct spring_electrical_control_struct ctrl0;
 #ifdef TIME
   clock_t  cpu;
 #endif
-  struct spring_electrical_control_struct ctrl0;
 
   ctrl0 = *ctrl;
 
@@ -1563,20 +2014,50 @@ void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0, spring_ele
   if (n <= 0 || dim <= 0) return;
 
   if (!SparseMatrix_is_symmetric(A, FALSE) || A->type != MATRIX_TYPE_REAL){
-    A = SparseMatrix_get_real_adjacency_matrix_symmetrized(A);
+    if (ctrl->method == METHOD_SPRING_MAXENT){
+      A = SparseMatrix_symmetrize_nodiag(A, FALSE);
+      assert(D0);
+      D = SparseMatrix_symmetrize_nodiag(D, FALSE);
+    } else {
+      A = SparseMatrix_get_real_adjacency_matrix_symmetrized(A);
+    }
   } else {
+    if (ctrl->method == METHOD_SPRING_MAXENT){
+      assert(D0);
+      D = SparseMatrix_remove_diagonal(D);
+    }
     A = SparseMatrix_remove_diagonal(A);
   }
 
-  mctrl = Multilevel_control_new();
+  /* we first generate a layout discarding (shorting) the edge labels nodes, then assign the edge label nodes at the average of their neighbors */
+  if ((ctrl->edge_labeling_scheme == ELSCHEME_STRAIGHTLINE_PENALTY || ctrl->edge_labeling_scheme == ELSCHEME_STRAIGHTLINE_PENALTY2)
+      && n_edge_label_nodes > 0){
+    SparseMatrix A2;
+
+    real *x2 = MALLOC(sizeof(real)*(A->m)*dim);
+    A2 = shorting_edge_label_nodes(A, n_edge_label_nodes, edge_label_nodes);
+    multilevel_spring_electrical_embedding(dim, A2, NULL, ctrl, NULL, NULL, x2, 0, NULL, flag);
+
+    assert(!(*flag));
+    attach_edge_label_coordinates(dim, A, n_edge_label_nodes, edge_label_nodes, x, x2);
+    remove_overlap(dim, A, x, label_sizes, ctrl->overlap, ctrl->initial_scaling,
+		   ctrl->edge_labeling_scheme, n_edge_label_nodes, edge_label_nodes, A, flag);
+    SparseMatrix_delete(A2);
+    FREE(x2);
+    if (A != A0) SparseMatrix_delete(A);
+
+    return;
+  }
+
+  mctrl = Multilevel_control_new(ctrl->multilevel_coarsen_scheme, ctrl->multilevel_coarsen_mode);
   mctrl->maxlevel = ctrl->multilevels;
-  grid0 = Multilevel_new(A, node_weights, mctrl);
+  grid0 = Multilevel_new(A, D, node_weights, mctrl);
 
   grid = Multilevel_get_coarsest(grid0);
   if (Multilevel_is_finest(grid)){
     xc = x;
   } else {
-    xc = N_GNEW(grid->n*dim,real);
+    xc = MALLOC(sizeof(real)*grid->n*dim);
   }
 
   plg = power_law_graph(A);
@@ -1596,12 +2077,42 @@ void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0, spring_ele
       }
     }
 #endif
-    if (ctrl->tscheme == QUAD_TREE_NONE){
-      spring_electrical_embedding_slow(dim, grid->A, ctrl, grid->node_weights, xc, flag);
-    } else if (ctrl->tscheme == QUAD_TREE_NORMAL){
-      spring_electrical_embedding(dim, grid->A, ctrl, grid->node_weights, xc, flag);
-    } else if (ctrl->tscheme == QUAD_TREE_FAST){
-      spring_electrical_embedding_fast(dim, grid->A, ctrl, grid->node_weights, xc, flag);
+    if (ctrl->method == METHOD_SPRING_ELECTRICAL){
+      if (ctrl->tscheme == QUAD_TREE_NONE){
+	spring_electrical_embedding_slow(dim, grid->A, ctrl, grid->node_weights, xc, flag);
+      } else if (ctrl->tscheme == QUAD_TREE_FAST || (ctrl->tscheme == QUAD_TREE_HYBRID && grid->A->m > QUAD_TREE_HYBRID_SIZE)){
+	if (ctrl->tscheme == QUAD_TREE_HYBRID && grid->A->m > 10 && Verbose){
+	  fprintf(stderr, "QUAD_TREE_HYBRID, size larger than %d, switch to fast quadtree", QUAD_TREE_HYBRID_SIZE);
+	}
+	spring_electrical_embedding_fast(dim, grid->A, ctrl, grid->node_weights, xc, flag);
+      } else if (ctrl->tscheme == QUAD_TREE_NORMAL){
+	spring_electrical_embedding(dim, grid->A, ctrl, grid->node_weights, xc, flag);
+      } else {
+	spring_electrical_embedding(dim, grid->A, ctrl, grid->node_weights, xc, flag);
+      }
+    } else if (ctrl->method == METHOD_SPRING_MAXENT){
+      double rho = 0.05;
+
+      ctrl->step = 1;
+      ctrl->adaptive_cooling = TRUE;
+      if (Multilevel_is_coarsest(grid)){
+	ctrl->maxiter=500;
+	rho = 0.5;
+      } else {
+	ctrl->maxiter=100;
+      }
+
+      if (Multilevel_is_finest(grid)) {/* gradually reduce influence of entropy */
+	spring_maxent_embedding(dim, grid->A, grid->D, ctrl, grid->node_weights, xc, rho, flag);
+	ctrl->random_start = FALSE;
+	ctrl->step = .05;
+	ctrl->adaptive_cooling = FALSE;
+	spring_maxent_embedding(dim, grid->A, grid->D, ctrl, grid->node_weights, xc, rho/2, flag);
+	spring_maxent_embedding(dim, grid->A, grid->D, ctrl, grid->node_weights, xc, rho/8, flag);
+	spring_maxent_embedding(dim, grid->A, grid->D, ctrl, grid->node_weights, xc, rho/32, flag);
+      } else {
+	spring_maxent_embedding(dim, grid->A, grid->D, ctrl, grid->node_weights, xc, rho, flag);
+      }
     } else {
       assert(0);
     }
@@ -1616,7 +2127,7 @@ void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0, spring_ele
     if (Multilevel_is_finest(grid)){
       xf = x;
     } else {
-      xf = N_GNEW(grid->n*dim,real);
+      xf = MALLOC(sizeof(real)*grid->n*dim);
     }
     prolongate(dim, grid->A, P, grid->R, xc, xf, coarsen_scheme_used, (ctrl->K)*0.001);
     FREE(xc);
@@ -1643,18 +2154,68 @@ void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0, spring_ele
   if (dim == 2){
     pcp_rotate(n, dim, x);
   }
+  if (ctrl->rotation != 0) rotate(n, dim, x, ctrl->rotation);
 
-  if (Verbose) 
-    fprintf(stderr, "sfdp: overlap=%d scaling %.02f\n",
-      ctrl->overlap, ctrl->initial_scaling);
+  if (Verbose) fprintf(stderr, "ctrl->overlap=%d\n",ctrl->overlap);
 
-  if (ctrl->overlap >= 0)
-    remove_overlap(dim, A, A->m, x, label_sizes, ctrl->overlap, ctrl->initial_scaling, flag);
+  remove_overlap(dim, A, x, label_sizes, ctrl->overlap, ctrl->initial_scaling,
+		 ctrl->edge_labeling_scheme, n_edge_label_nodes, edge_label_nodes, A, flag);
 
  RETURN:
   *ctrl = ctrl0;
   if (A != A0) SparseMatrix_delete(A);
+  if (D && D != D0) SparseMatrix_delete(D);
   Multilevel_control_delete(mctrl);
   Multilevel_delete(grid0);
 }
 
+#ifdef GVIEWER
+struct multilevel_spring_electrical_embedding_data {
+  int dim;
+  SparseMatrix A;
+  SparseMatrix D;
+  spring_electrical_control ctrl;
+  real *node_weights;
+  real *label_sizes;
+  real *x;
+  int n_edge_label_nodes;
+  int *edge_label_nodes;
+  int *flag;
+};
+
+void multilevel_spring_electrical_embedding_gv(void* data){
+  struct multilevel_spring_electrical_embedding_data* d;
+
+  d = (struct multilevel_spring_electrical_embedding_data*) data;
+  multilevel_spring_electrical_embedding_core(d->dim, d->A, d->D, d->ctrl, d->node_weights, d->label_sizes, d->x, d->n_edge_label_nodes, d->edge_label_nodes, d->flag);
+  gviewer_reset_graph_coord(d->A, d->dim, d->x);/* A inside spring_electrical gets deleted */
+}
+void multilevel_spring_electrical_embedding(int dim, SparseMatrix A, SparseMatrix D, spring_electrical_control ctrl, real *node_weights, real *label_sizes, 
+				 real *x, int n_edge_label_nodes, int *edge_label_nodes, int *flag){
+  struct multilevel_spring_electrical_embedding_data data = {dim, A, D, ctrl, node_weights, label_sizes, x, n_edge_label_nodes, edge_label_nodes, flag};
+
+  int argcc = 1;
+  char **argvv;
+
+  if (!Gviewer) return multilevel_spring_electrical_embedding_core(dim, A, D, ctrl, node_weights, label_sizes, x, n_edge_label_nodes, edge_label_nodes, flag);
+
+  argcc = 1;
+  argvv = malloc(sizeof(char*)*argcc);
+  argvv[0] = malloc(sizeof(char));
+  argvv[0][0] = '1';
+
+  gviewer_set_edge_color_scheme(COLOR_SCHEME_NO);
+  //gviewer_set_edge_color_scheme(COLOR_SCHEME_MEDIAN_AS_GREEN);
+  gviewer_toggle_bgcolor();
+  //gviewer_toggle_vertex();
+  //gviewer_init(&argcc, argvv, 0.01, 20, 60, 2*1010, 2*770, A, dim, x, &(data), multilevel_spring_electrical_embedding_gv);
+  gviewer_init(&argcc, argvv, 0.01, 20, 60, 320, 320, A, dim, x, &(data), multilevel_spring_electrical_embedding_gv);
+  free(argvv);
+
+}
+#else
+void multilevel_spring_electrical_embedding(int dim, SparseMatrix A, SparseMatrix D, spring_electrical_control ctrl, real *node_weights, real *label_sizes, 
+				 real *x, int n_edge_label_nodes, int *edge_label_nodes, int *flag){
+  return multilevel_spring_electrical_embedding_core(dim, A, D, ctrl, node_weights, label_sizes, x, n_edge_label_nodes, edge_label_nodes, flag);
+}
+#endif
