@@ -1,4 +1,3 @@
-/* $Id: xdot.c,v 1.22 2011/01/26 23:52:19 erg Exp $Revision: */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -19,6 +18,76 @@
 #define NEW(t)           (t*)calloc(1, sizeof(t))
 #define N_NEW(n,t)       (t*)calloc((n), sizeof(t))
 
+typedef struct {
+    unsigned char *buf;		/* start of buffer */
+    unsigned char *ptr;		/* next place to write */
+    unsigned char *eptr;	/* end of buffer */
+    int dyna;			/* true if buffer is malloc'ed */
+} agxbuf;
+
+#define agxbputc(X,C) ((((X)->ptr >= (X)->eptr) ? agxbmore(X,1) : 0), \
+          (int)(*(X)->ptr++ = ((unsigned char)C)))
+#define agxbuse(X) (agxbputc(X,'\0'),(char*)((X)->ptr = (X)->buf))
+
+static void agxbinit(agxbuf * xb, unsigned int hint, unsigned char *init)
+{
+    if (init) {
+	xb->buf = init;
+	xb->dyna = 0;
+    } else {
+	if (hint == 0)
+	    hint = BUFSIZ;
+	xb->dyna = 1;
+	xb->buf = N_NEW(hint, unsigned char);
+    }
+    xb->eptr = xb->buf + hint;
+    xb->ptr = xb->buf;
+    *xb->ptr = '\0';
+}
+static int agxbmore(agxbuf * xb, unsigned int ssz)
+{
+    int cnt;			/* current no. of characters in buffer */
+    int size;			/* current buffer size */
+    int nsize;			/* new buffer size */
+    unsigned char *nbuf;	/* new buffer */
+
+    size = xb->eptr - xb->buf;
+    nsize = 2 * size;
+    if (size + ssz > nsize)
+	nsize = size + ssz;
+    cnt = xb->ptr - xb->buf;
+    if (xb->dyna) {
+	nbuf = realloc(xb->buf, nsize);
+    } else {
+	nbuf = N_NEW(nsize, unsigned char);
+	memcpy(nbuf, xb->buf, cnt);
+	xb->dyna = 1;
+    }
+    xb->buf = nbuf;
+    xb->ptr = xb->buf + cnt;
+    xb->eptr = xb->buf + nsize;
+    return 0;
+}
+
+static int agxbput(char *s, agxbuf * xb)
+{
+    unsigned int ssz = strlen(s);
+    if (xb->ptr + ssz > xb->eptr)
+	agxbmore(xb, ssz);
+    memcpy(xb->ptr, s, ssz);
+    xb->ptr += ssz;
+    return ssz;
+}
+
+/* agxbfree:
+ * Free any malloced resources.
+ */
+static void agxbfree(agxbuf * xb)
+{
+    if (xb->dyna)
+	free(xb->buf);
+}
+
 /* the parse functions should return NULL on error */
 static char *parseReal(char *s, double *fp)
 {
@@ -177,6 +246,9 @@ static char *parseAlign(char *s, xdot_align * ap)
 
 static char *parseOp(xdot_op * op, char *s, drawfunc_t ops[], int* error)
 {
+    char* cs;
+    xdot_color clr;
+
     *error = 0;
     while (isspace(*s))
 	s++;
@@ -230,19 +302,41 @@ static char *parseOp(xdot_op * op, char *s, drawfunc_t ops[], int* error)
 	break;
 
     case 'c':
-	op->kind = xd_pen_color;
-	s = parseString(s, &op->u.color);
+	s = parseString(s, &cs);
 	CHK(s);
-	if (ops)
-	    op->drawfunc = ops[xop_pen_color];
+	cs = parseXDotColor (cs, &clr);
+	CHK(cs);
+	if (clr.type == xd_none) {
+	    op->kind = xd_pen_color;
+	    op->u.color = clr.u.clr;
+	    if (ops)
+		op->drawfunc = ops[xop_pen_color];
+	}
+	else {
+	    op->kind = xd_grad_pen_color;
+	    op->u.grad_color = clr;
+	    if (ops)
+		op->drawfunc = ops[xop_grad_pen_color];
+	}
 	break;
 
     case 'C':
-	op->kind = xd_fill_color;
-	s = parseString(s, &op->u.color);
+	s = parseString(s, &cs);
 	CHK(s);
-	if (ops)
-	    op->drawfunc = ops[xop_fill_color];
+	cs = parseXDotColor (cs, &clr);
+	CHK(cs);
+	if (clr.type == xd_none) {
+	    op->kind = xd_fill_color;
+	    op->u.color = clr.u.clr;
+	    if (ops)
+		op->drawfunc = ops[xop_fill_color];
+	}
+	else {
+	    op->kind = xd_grad_fill_color;
+	    op->u.grad_color = clr;
+	    if (ops)
+		op->drawfunc = ops[xop_grad_fill_color];
+	}
 	break;
 
     case 'L':
@@ -386,11 +480,41 @@ xdot *parseXDot(char *s)
 
 typedef void (*pf) (char *, void *);
 
+/* trim:
+ * Trailing zeros are removed and decimal point, if possible.
+ */
+static void trim (char* buf)
+{
+    char* dotp;
+    char* p;
+
+    if ((dotp = strchr (buf,'.'))) {
+        p = dotp+1;
+        while (*p) p++;  // find end of string
+        p--;
+        while (*p == '0') *p-- = '\0';
+        if (*p == '.')        // If all decimals were zeros, remove ".".
+            *p = '\0';
+        else
+            p++;
+    }
+}
+
 static void printRect(xdot_rect * r, pf print, void *info)
 {
     char buf[128];
 
-    sprintf(buf, " %.06f %.06f %.06f %.06f", r->x, r->y, r->w, r->h);
+    sprintf(buf, " %.02f", r->x);
+    trim(buf);
+    print(buf, info);
+    sprintf(buf, " %.02f", r->y);
+    trim(buf);
+    print(buf, info);
+    sprintf(buf, " %.02f", r->w);
+    trim(buf);
+    print(buf, info);
+    sprintf(buf, " %.02f", r->h);
+    trim(buf);
     print(buf, info);
 }
 
@@ -402,7 +526,11 @@ static void printPolyline(xdot_polyline * p, pf print, void *info)
     sprintf(buf, " %d", p->cnt);
     print(buf, info);
     for (i = 0; i < p->cnt; i++) {
-	sprintf(buf, " %.06f %.06f", p->pts[i].x, p->pts[i].y);
+	sprintf(buf, " %.02f", p->pts[i].x);
+	trim(buf);
+	print(buf, info);
+	sprintf(buf, " %.02f", p->pts[i].y);
+	trim(buf);
 	print(buf, info);
     }
 }
@@ -424,11 +552,15 @@ static void printInt(int i, pf print, void *info)
     print(buf, info);
 }
 
-static void printFloat(float f, pf print, void *info)
+static void printFloat(float f, pf print, void *info, int space)
 {
     char buf[128];
 
-    sprintf(buf, " %f", f);
+    if (space)
+	sprintf(buf, " %.02f", f);
+    else
+	sprintf(buf, "%.02f", f);
+    trim (buf);
     print(buf, info);
 }
 
@@ -447,8 +579,58 @@ static void printAlign(xdot_align a, pf print, void *info)
     }
 }
 
-static void printXDot_Op(xdot_op * op, pf print, void *info)
+static void
+gradprint (char* s, void* v)
 {
+    agxbput(s, (agxbuf*)v);
+}
+
+static void
+toGradString (agxbuf* xb, xdot_color* cp)
+{
+    int i, n_stops;
+    xdot_color_stop* stops;
+
+    if (cp->type == xd_linear) {
+	agxbputc (xb, '[');
+	printFloat (cp->u.ling.x0, gradprint, xb, 0);
+	printFloat (cp->u.ling.y0, gradprint, xb, 1);
+	printFloat (cp->u.ling.x1, gradprint, xb, 1);
+	printFloat (cp->u.ling.y1, gradprint, xb, 1);
+	n_stops = cp->u.ling.n_stops;
+	stops = cp->u.ling.stops;
+    }
+    else {
+	agxbputc (xb, '(');
+	printFloat (cp->u.ring.x0, gradprint, xb, 0);
+	printFloat (cp->u.ring.y0, gradprint, xb, 1);
+	printFloat (cp->u.ring.r0, gradprint, xb, 1);
+	printFloat (cp->u.ring.x1, gradprint, xb, 1);
+	printFloat (cp->u.ring.y1, gradprint, xb, 1);
+	printFloat (cp->u.ring.r1, gradprint, xb, 1);
+	n_stops = cp->u.ring.n_stops;
+	stops = cp->u.ring.stops;
+    }
+    printInt (n_stops, gradprint, xb);
+    for (i = 0; i < n_stops; i++) {
+	printFloat (stops[i].frac, gradprint, xb, 1);
+	printString (stops[i].color, gradprint, xb);
+    }
+
+    if (cp->type == xd_linear)
+	agxbputc (xb, ']');
+    else
+	agxbputc (xb, ')');
+}
+
+typedef void (*print_op)(xdot_op * op, pf print, void *info, int more);
+
+static void printXDot_Op(xdot_op * op, pf print, void *info, int more)
+{
+    agxbuf xb;
+    unsigned char buf[BUFSIZ];
+
+    agxbinit (&xb, BUFSIZ, buf);
     switch (op->kind) {
     case xd_filled_ellipse:
 	print("E", info);
@@ -479,10 +661,20 @@ static void printXDot_Op(xdot_op * op, pf print, void *info)
 	print("c", info);
 	printString(op->u.color, print, info);
 	break;
+    case xd_grad_pen_color:
+	print("c", info);
+	toGradString (&xb, &op->u.grad_color);
+	printString(agxbuse(&xb), print, info);
+	break;
     case xd_fill_color:
 	print("C", info);
 	printString(op->u.color, print, info);
 	break;
+    case xd_grad_fill_color:
+	print("C", info);
+	toGradString (&xb, &op->u.grad_color);
+	printString(agxbuse(&xb), print, info);
+	break;
     case xd_polyline:
 	print("L", info);
 	printPolyline(&op->u.polyline, print, info);
@@ -498,7 +690,7 @@ static void printXDot_Op(xdot_op * op, pf print, void *info)
     case xd_font:
 	print("F", info);
 	op->kind = xd_font;
-	printFloat(op->u.font.size, print, info);
+	printFloat(op->u.font.size, print, info, 1);
 	printString(op->u.font.name, print, info);
 	break;
     case xd_style:
@@ -511,98 +703,162 @@ static void printXDot_Op(xdot_op * op, pf print, void *info)
 	printString(op->u.image.name, print, info);
 	break;
     }
+    if (more)
+	print(" ", info);
+    agxbfree (&xb);
 }
 
-static void _printXDot(xdot * x, pf print, void *info)
+static void jsonRect(xdot_rect * r, pf print, void *info)
+{
+    char buf[128];
+
+    sprintf(buf, "[%.06f,%.06f,%.06f,%.06f]", r->x, r->y, r->w, r->h);
+    print(buf, info);
+}
+
+static void jsonPolyline(xdot_polyline * p, pf print, void *info)
+{
+    int i;
+    char buf[128];
+
+    print("[", info);
+    for (i = 0; i < p->cnt; i++) {
+	sprintf(buf, "%.06f,%.06f", p->pts[i].x, p->pts[i].y);
+	print(buf, info);
+	if (i < p->cnt-1) print(",", info);
+    }
+    print("]", info);
+}
+
+static void jsonString(char *p, pf print, void *info)
+{
+    unsigned char c, buf[BUFSIZ];
+    agxbuf xb;
+
+    agxbinit(&xb, BUFSIZ, buf);
+    agxbputc(&xb, '"');
+    while ((c = *p++)) {
+	if (c == '"') agxbput("\\\"", &xb);
+	else if (c == '\\') agxbput("\\\\", &xb);
+	/* else if (c > 127) handle UTF-8 */
+	else agxbputc(&xb, c);
+    }
+    agxbputc(&xb, '"');
+    print(agxbuse(&xb), info);
+    agxbfree(&xb);
+}
+
+static void jsonXDot_Op(xdot_op * op, pf print, void *info, int more)
+{
+    agxbuf xb;
+    unsigned char buf[BUFSIZ];
+
+    agxbinit (&xb, BUFSIZ, buf);
+    switch (op->kind) {
+    case xd_filled_ellipse:
+	print("{E : ", info);
+	jsonRect(&op->u.ellipse, print, info);
+	break;
+    case xd_unfilled_ellipse:
+	print("{e : ", info);
+	jsonRect(&op->u.ellipse, print, info);
+	break;
+    case xd_filled_polygon:
+	print("{P : ", info);
+	jsonPolyline(&op->u.polygon, print, info);
+	break;
+    case xd_unfilled_polygon:
+	print("{p : ", info);
+	jsonPolyline(&op->u.polygon, print, info);
+	break;
+    case xd_filled_bezier:
+	print("{b : ", info);
+	jsonPolyline(&op->u.bezier, print, info);
+	break;
+    case xd_unfilled_bezier:
+	print("{B : ", info);
+	jsonPolyline(&op->u.bezier, print, info);
+	break;
+    case xd_pen_color:
+	print("{c : ", info);
+	jsonString(op->u.color, print, info);
+	break;
+    case xd_grad_pen_color:
+	print("{c : ", info);
+	toGradString (&xb, &op->u.grad_color);
+	jsonString(agxbuse(&xb), print, info);
+	break;
+    case xd_fill_color:
+	print("{C : ", info);
+	jsonString(op->u.color, print, info);
+	break;
+    case xd_grad_fill_color:
+	print("{C : ", info);
+	toGradString (&xb, &op->u.grad_color);
+	jsonString(agxbuse(&xb), print, info);
+	break;
+    case xd_polyline:
+	print("{L :", info);
+	jsonPolyline(&op->u.polyline, print, info);
+	break;
+    case xd_text:
+	print("{T : [", info);
+	printInt(op->u.text.x, print, info);
+	print(",", info);
+	printInt(op->u.text.y, print, info);
+	print(",", info);
+	printAlign(op->u.text.align, print, info);
+	print(",", info);
+	printInt(op->u.text.width, print, info);
+	print(",", info);
+	jsonString(op->u.text.text, print, info);
+	print("]", info);
+	break;
+    case xd_font:
+	print("{F : [", info);
+	op->kind = xd_font;
+	printFloat(op->u.font.size, print, info, 1);
+	print(",", info);
+	jsonString(op->u.font.name, print, info);
+	print("]", info);
+	break;
+    case xd_style:
+	print("{S : ", info);
+	jsonString(op->u.style, print, info);
+	break;
+    case xd_image:
+	print("{I : [", info);
+	jsonRect(&op->u.image.pos, print, info);
+	print(",", info);
+	jsonString(op->u.image.name, print, info);
+	print("]", info);
+	break;
+    }
+    if (more)
+	print("},\n", info);
+    else
+	print("}\n", info);
+    agxbfree (&xb);
+}
+
+static void _printXDot(xdot * x, pf print, void *info, print_op ofn)
 {
     int i;
     xdot_op *op;
     char *base = (char *) (x->ops);
     for (i = 0; i < x->cnt; i++) {
 	op = (xdot_op *) (base + i * x->sz);
-	printXDot_Op(op, print, info);
-	if (i < x->cnt - 1)
-	    print(" ", info);
+	ofn(op, print, info, (i < x->cnt - 1));
     }
 }
 
-typedef struct {
-    unsigned char *buf;		/* start of buffer */
-    unsigned char *ptr;		/* next place to write */
-    unsigned char *eptr;	/* end of buffer */
-    int dyna;			/* true if buffer is malloc'ed */
-} agxbuf;
-
-#define agxbputc(X,C) ((((X)->ptr >= (X)->eptr) ? agxbmore(X,1) : 0), \
-          (int)(*(X)->ptr++ = ((unsigned char)C)))
-#define agxbuse(X) (agxbputc(X,'\0'),(char*)((X)->ptr = (X)->buf))
-
-static void agxbinit(agxbuf * xb, unsigned int hint, unsigned char *init)
-{
-    if (init) {
-	xb->buf = init;
-	xb->dyna = 0;
-    } else {
-	if (hint == 0)
-	    hint = BUFSIZ;
-	xb->dyna = 1;
-	xb->buf = N_NEW(hint, unsigned char);
-    }
-    xb->eptr = xb->buf + hint;
-    xb->ptr = xb->buf;
-    *xb->ptr = '\0';
-}
-static int agxbmore(agxbuf * xb, unsigned int ssz)
-{
-    int cnt;			/* current no. of characters in buffer */
-    int size;			/* current buffer size */
-    int nsize;			/* new buffer size */
-    unsigned char *nbuf;	/* new buffer */
-
-    size = xb->eptr - xb->buf;
-    nsize = 2 * size;
-    if (size + ssz > nsize)
-	nsize = size + ssz;
-    cnt = xb->ptr - xb->buf;
-    if (xb->dyna) {
-	nbuf = realloc(xb->buf, nsize);
-    } else {
-	nbuf = N_NEW(nsize, unsigned char);
-	memcpy(nbuf, xb->buf, cnt);
-	xb->dyna = 1;
-    }
-    xb->buf = nbuf;
-    xb->ptr = xb->buf + cnt;
-    xb->eptr = xb->buf + nsize;
-    return 0;
-}
-
-static int agxbput(char *s, agxbuf * xb)
-{
-    unsigned int ssz = strlen(s);
-    if (xb->ptr + ssz > xb->eptr)
-	agxbmore(xb, ssz);
-    memcpy(xb->ptr, s, ssz);
-    xb->ptr += ssz;
-    return ssz;
-}
-
-/* agxbfree:
- * Free any malloced resources.
- */
-static void agxbfree(agxbuf * xb)
-{
-    if (xb->dyna)
-	free(xb->buf);
-}
-
 char *sprintXDot(xdot * x)
 {
     char *s;
     unsigned char buf[BUFSIZ];
     agxbuf xb;
     agxbinit(&xb, BUFSIZ, buf);
-    _printXDot(x, (pf) agxbput, &xb);
+    _printXDot(x, (pf) agxbput, &xb, printXDot_Op);
     s = strdup(agxbuse(&xb));
     agxbfree(&xb);
 
@@ -611,7 +867,14 @@ char *sprintXDot(xdot * x)
 
 void fprintXDot(FILE * fp, xdot * x)
 {
-    _printXDot(x, (pf) fputs, fp);
+    _printXDot(x, (pf) fputs, fp, printXDot_Op);
+}
+
+void jsonXDot(FILE * fp, xdot * x)
+{
+    fputs ("[\n", fp);
+    _printXDot(x, (pf) fputs, fp, jsonXDot_Op);
+    fputs ("]\n", fp);
 }
 
 static void freeXOpData(xdot_op * x)
@@ -635,6 +898,10 @@ static void freeXOpData(xdot_op * x)
     case xd_pen_color:
 	free(x->u.color);
 	break;
+    case xd_grad_fill_color:
+    case xd_grad_pen_color:
+	freeXDotColor (&x->u.grad_color);
+	break;
     case xd_font:
 	free(x->u.font.name);
 	break;
@@ -722,6 +989,157 @@ int statXDot (xdot* x, xdot_stats* sp)
     return 0;
 }
 
+xdot_grad_type 
+colorType (char* cp)
+{
+    xdot_grad_type rv;
+
+    switch (*cp) {
+    case '[' :
+	rv = xd_linear;
+	break;
+    case '(' :
+	rv = xd_radial;
+	break;
+    default :
+	rv = xd_none;
+	break;
+    }
+    return rv;
+}
+
+#define CHK1(s) if(!s){free(stops);return NULL;}
+
+/* radGradient:
+ * Parse radial gradient spec
+ * Return NULL on failure.
+ */
+static char*
+radGradient (char* cp, xdot_color* clr)
+{
+    char* s = cp;
+    int i;
+    double d;
+    xdot_color_stop* stops = NULL;
+
+    clr->type = xd_radial;
+    s = parseReal(s, &clr->u.ring.x0);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ring.y0);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ring.r0);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ring.x1);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ring.y1);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ring.r1);
+    CHK1(s);
+    s = parseInt(s, &clr->u.ring.n_stops);
+    CHK1(s);
+
+    stops = N_NEW(clr->u.ring.n_stops,xdot_color_stop);
+    for (i = 0; i < clr->u.ring.n_stops; i++) {
+	s = parseReal(s, &d);
+	CHK1(s);
+	stops[i].frac = d;
+	s = parseString(s, &stops[i].color);
+	CHK1(s);
+    }
+    clr->u.ring.stops = stops;
+
+    return cp;
+}
+
+/* linGradient:
+ * Parse linear gradient spec
+ * Return NULL on failure.
+ */
+static char*
+linGradient (char* cp, xdot_color* clr)
+{
+    char* s = cp;
+    int i;
+    double d;
+    xdot_color_stop* stops = NULL;
+
+    clr->type = xd_linear;
+    s = parseReal(s, &clr->u.ling.x0);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ling.y0);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ling.x1);
+    CHK1(s);
+    s = parseReal(s, &clr->u.ling.y1);
+    CHK1(s);
+    s = parseInt(s, &clr->u.ling.n_stops);
+    CHK1(s);
+
+    stops = N_NEW(clr->u.ling.n_stops,xdot_color_stop);
+    for (i = 0; i < clr->u.ling.n_stops; i++) {
+	s = parseReal(s, &d);
+	CHK1(s);
+	stops[i].frac = d;
+	s = parseString(s, &stops[i].color);
+	CHK1(s);
+    }
+    clr->u.ling.stops = stops;
+
+    return cp;
+}
+
+/* parseXDotColor:
+ * Parse xdot color spec: ordinary or gradient
+ * The result is stored in clr.
+ * Return NULL on failure.
+ */
+char*
+parseXDotColor (char* cp, xdot_color* clr)
+{
+    char c = *cp;
+
+    switch (c) {
+    case '[' :
+	return linGradient (cp+1, clr);
+	break;
+    case '(' :
+	return radGradient (cp+1, clr);
+	break;
+    case '#' :
+    case '/' :
+	clr->type = xd_none; 
+	clr->u.clr = cp;
+	return cp;
+	break;
+    default :
+	if (isalnum(c)) {
+	    clr->type = xd_none; 
+	    clr->u.clr = cp;
+	    return cp;
+	}
+	else
+	    return NULL;
+    }
+}
+
+void freeXDotColor (xdot_color* cp)
+{
+    int i;
+
+    if (cp->type == xd_linear) {
+	for (i = 0; i < cp->u.ling.n_stops; i++) {
+	    free (cp->u.ling.stops[i].color);
+	}
+	free (cp->u.ling.stops);
+    }
+    else if (cp->type == xd_radial) {
+	for (i = 0; i < cp->u.ring.n_stops; i++) {
+	    free (cp->u.ring.stops[i].color);
+	}
+	free (cp->u.ring.stops);
+    }
+}
+
 #if 0
 static void execOp(xdot_op * op, int param)
 {
