@@ -1,17 +1,14 @@
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2007 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #ifndef XDOT_H
 #define XDOT_H
@@ -21,6 +18,40 @@
 #endif
 #define INITIAL_XDOT_CAPACITY 512
 
+typedef enum {
+    xd_none,
+    xd_linear,
+    xd_radial
+} xdot_grad_type;
+
+typedef struct {
+    float frac;
+    char* color;
+} xdot_color_stop;
+
+typedef struct {
+    double x0, y0;
+    double x1, y1;
+    int n_stops;
+    xdot_color_stop* stops;
+} xdot_linear_grad;
+
+typedef struct {
+    double x0, y0, r0;
+    double x1, y1, r1;
+    int n_stops;
+    xdot_color_stop* stops;
+} xdot_radial_grad;
+
+typedef struct {
+    xdot_grad_type type;
+    union {
+	char* clr;
+	xdot_linear_grad ling;
+	xdot_radial_grad ring;
+    } u;
+} xdot_color;
+
 typedef enum {
     xd_left, xd_center, xd_right
 } xdot_align;
@@ -60,7 +91,8 @@ typedef enum {
     xd_filled_polygon, xd_unfilled_polygon,
     xd_filled_bezier,  xd_unfilled_bezier,
     xd_polyline,       xd_text,
-    xd_fill_color,     xd_pen_color, xd_font, xd_style, xd_image
+    xd_fill_color,     xd_pen_color, xd_font, xd_style, xd_image,
+    xd_grad_fill_color,     xd_grad_pen_color
 } xdot_kind; 
     
 typedef enum {
@@ -68,7 +100,8 @@ typedef enum {
     xop_polygon,
     xop_bezier,
     xop_polyline,       xop_text,
-    xop_fill_color,     xop_pen_color, xop_font, xop_style, xop_image
+    xop_fill_color,     xop_pen_color, xop_font, xop_style, xop_image,
+    xop_grad_fill_color,     xop_grad_pen_color
 } xop_kind; 
     
 typedef struct _xdot_op xdot_op;
@@ -85,6 +118,7 @@ struct _xdot_op {
       xdot_text text;          /* xd_text */
       xdot_image image;        /* xd_image */
       char* color;             /* xd_fill_color, xd_pen_color */
+      xdot_color grad_color;   /* xd_grad_fill_color, xd_grad_pen_color */
       xdot_font font;          /* xd_font */
       char* style;             /* xd_style */
     } u;
@@ -94,19 +128,40 @@ struct _xdot_op {
 #define XDOT_PARSE_ERROR 1
 
 typedef struct {
-    int cnt;
-    int sz;
+    int cnt;  /* no. of xdot ops */
+    int sz;   /* sizeof structure containing xdot_op as first field */
     xdot_op* ops;
     freefunc_t freefunc;
     int flags;
 } xdot;
 
+typedef struct {
+    int cnt;  /* no. of xdot ops */
+    int n_ellipse;
+    int n_polygon;
+    int n_polygon_pts;
+    int n_polyline;
+    int n_polyline_pts;
+    int n_bezier;
+    int n_bezier_pts;
+    int n_text;
+    int n_font;
+    int n_style;
+    int n_color;
+    int n_image;
+} xdot_stats;
+
 /* ops are indexed by xop_kind */
 extern xdot* parseXDotF (char*, drawfunc_t opfns[], int sz);
 extern xdot* parseXDotFOn (char*, drawfunc_t opfns[], int sz, xdot*);
 extern xdot* parseXDot (char*);
 extern char* sprintXDot (xdot*);
 extern void fprintXDot (FILE*, xdot*);
+extern void jsonXDot (FILE*, xdot*);
 extern void freeXDot (xdot*);
+extern int statXDot (xdot*, xdot_stats*);
+extern xdot_grad_type colorTypeXDot (char*);
+extern char* parseXDotColor (char* cp, xdot_color* clr);
+extern void freeXDotColor (xdot_color*);
 
 #endif
