@@ -1,18 +1,15 @@
-/* $Id: gv.cpp,v 1.63 2009/07/16 15:53:10 ellson Exp $ $Revision: 1.63 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #include <string.h>
 #include <stdlib.h>
@@ -23,6 +20,13 @@ extern void gv_string_writer_init(GVC_t *gvc);
 extern void gv_channel_writer_init(GVC_t *gvc);
 }
 
+#ifdef WITH_CGRAPH
+#define agfindattr(x,s) agattrsym(x,s)
+#define agraphattr(g,n,s) agattr(g,AGRAPH,n,s)
+#define agnodeattr(g,n,s) agattr(g,AGNODE,n,s)
+#define agedgeattr(g,n,s) agattr(g,AGEDGE,n,s)
+#endif
+
 static char emptystring[] = {'\0'};
 
 static GVC_t *gvc;
@@ -36,28 +40,44 @@ Agraph_t *graph(char *name)
 {
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    return agopen(name, Agundirected, 0);
+#else
     return agopen(name, AGRAPH);
+#endif
 }
 
 Agraph_t *digraph(char *name)
 {
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    return agopen(name, Agdirected, 0);
+#else
     return agopen(name, AGDIGRAPH);
+#endif
 }
 
 Agraph_t *strictgraph(char *name)
 {
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    return agopen(name, Agstrictundirected, 0);
+#else
     return agopen(name, AGRAPHSTRICT);
+#endif
 }
 
 Agraph_t *strictdigraph(char *name)
 {
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    return agopen(name, Agstrictdirected, 0);
+#else
     return agopen(name, AGDIGRAPHSTRICT);
+#endif
 }
 
 Agraph_t *readstring(char *string)
@@ -71,7 +91,11 @@ Agraph_t *read(FILE *f)
 {
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    return agread(f, NULL);
+#else
     return agread(f);
+#endif
 }
 
 Agraph_t *read(const char *filename)
@@ -84,7 +108,11 @@ Agraph_t *read(const char *filename)
         return NULL;
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    g = agread(f, NULL);
+#else
     g = agread(f);
+#endif
     fclose(f);
     return g;
 }
@@ -94,37 +122,63 @@ Agraph_t *graph(Agraph_t *g, char *name)
 {
     if (!gvc)
         gv_init();
+#ifdef WITH_CGRAPH
+    return agsubg(g, name, 1);
+#else
     return agsubg(g, name);
+#endif
 }
 
 Agnode_t *node(Agraph_t *g, char *name)
 {
+    if (!gvc)
+        return NULL;
+#ifdef WITH_CGRAPH
+    return agnode(g, name, 1);
+#else
     // creating a protonode is not permitted
-    if (!gvc || (name[0] == '\001' && strcmp (name, "\001proto") == 0))
+    if (name[0] == '\001' && strcmp (name, "\001proto") == 0)
         return NULL;
     return agnode(g, name);
+#endif
 }
 
 Agedge_t *edge(Agnode_t *t, Agnode_t *h)
 {
+    if (!gvc || !t || !h)
+        return NULL;
+#ifdef WITH_CGRAPH
     // edges from/to the protonode are not permitted
-    if (!gvc || !t || !h
-      || (agnameof(t)[0] == '\001' && strcmp (agnameof(t), "\001proto") == 0)
+    if (AGTYPE(t) == AGRAPH || AGTYPE(h) == AGRAPH)
+	return NULL;
+    return agedge(agraphof(t), t, h, NULL, 1);
+#else
+    // edges from/to the protonode are not permitted
+    if ((agnameof(t)[0] == '\001' && strcmp (agnameof(t), "\001proto") == 0)
       || (agnameof(h)[0] == '\001' && strcmp (agnameof(h), "\001proto") == 0))
         return NULL;
     return agedge(t->graph, t, h);
+#endif
 }
 
 // induce tail if necessary
 Agedge_t *edge(char *tname, Agnode_t *h)
 {
+#ifdef WITH_CGRAPH
+    return edge(node(agraphof(h), tname), h);
+#else
     return edge(node(h->graph, tname), h);
+#endif
 }
 
 // induce head if necessary
 Agedge_t *edge(Agnode_t *t, char *hname)
 {
+#ifdef WITH_CGRAPH
+    return edge(t, node(agraphof(t), hname));
+#else
     return edge(t, node(t->graph, hname));
+#endif
 }
 
 // induce tail/head if necessary
@@ -182,7 +236,11 @@ static void myagxset(void *obj, Agsym_t *a, char *val)
         if (val[len-1] == '>') {
             hs = strdup(val+1);
                 *(hs+len-2) = '\0';
+#ifdef WITH_CGRAPH
+            val = agstrdup_html(agraphof(obj),hs);
+#else
             val = agstrdup_html(hs);
+#endif
             free(hs);
         }
     }
@@ -214,6 +272,12 @@ char *setv(Agraph_t *g, char *attr, char *val)
 //-------------------------------------------------
 char *getv(Agnode_t *n, Agsym_t *a)
 {
+    if (!n || !a)
+        return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(n) == AGRAPH) // protonode   
+	return NULL;   // FIXME ??
+#endif
     return myagxget(n, a);
 }
 char *getv(Agnode_t *n, char *attr)
@@ -223,14 +287,26 @@ char *getv(Agnode_t *n, char *attr)
 
     if (!n || !attr)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(n) == AGRAPH) // protonode   
+	return NULL;   // FIXME ??
+#endif
     g = agroot(agraphof(n));
+#ifdef WITH_CGRAPH
+    a = agattr(g, AGNODE, attr, NULL);
+#else
     a = agfindattr(g->proto->n, attr);
+#endif
     return myagxget(n, a);
 }
 char *setv(Agnode_t *n, Agsym_t *a, char *val)
 {
     if (!n || !a || !val)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(n) == AGRAPH) // protonode   
+	return NULL;   // FIXME ??
+#endif
     myagxset(n, a, val);
     return val;
 }
@@ -241,8 +317,20 @@ char *setv(Agnode_t *n, char *attr, char *val)
 
     if (!n || !attr || !val)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(n) == AGRAPH) { // protonode   
+	g = (Agraph_t*)n;
+    	a = agattr(g, AGNODE, attr, val); // create default attribute in psuodo protonode
+	    // FIXME? - deal with html in "label" attributes
+	return val;
+    }
+#endif
     g = agroot(agraphof(n));
+#ifdef WITH_CGRAPH
+    a = agattr(g, AGNODE, attr, NULL);
+#else
     a = agfindattr(g->proto->n, attr);
+#endif
     if (!a)
         a = agnodeattr(g, attr, emptystring);
     myagxset(n, a, val);
@@ -251,6 +339,12 @@ char *setv(Agnode_t *n, char *attr, char *val)
 //-------------------------------------------------
 char *getv(Agedge_t *e, Agsym_t *a)
 {
+    if (!e || !a)
+        return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH) // protoedge   
+	return NULL;   // FIXME ??
+#endif
     return myagxget(e, a);
 }
 char *getv(Agedge_t *e, char *attr)
@@ -260,6 +354,10 @@ char *getv(Agedge_t *e, char *attr)
 
     if (!e || !attr)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH) // protoedge   
+	return NULL;   // FIXME ??
+#endif
     g = agraphof(agtail(e));
 #ifndef WITH_CGRAPH
     a = agfindattr(g->proto->e, attr);
@@ -272,6 +370,10 @@ char *setv(Agedge_t *e, Agsym_t *a, char *val)
 {
     if (!e || !a || !val)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH) // protoedge   
+	return NULL;   // FIXME ??
+#endif
     myagxset(e, a, val);
     return val;
 }
@@ -282,6 +384,14 @@ char *setv(Agedge_t *e, char *attr, char *val)
 
     if (!e || !attr || !val)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH) { // protoedge   
+	g = (Agraph_t*)e;
+    	a = agattr(g, AGEDGE, attr, val); // create default attribute in pseudo protoedge
+	    // FIXME? - deal with html in "label" attributes
+	return val;
+    }
+#endif
     g = agroot(agraphof(agtail(e)));
 #ifndef WITH_CGRAPH
     a = agfindattr(g->proto->e, attr);
@@ -322,6 +432,10 @@ Agedge_t *findedge(Agnode_t *t, Agnode_t *h)
 {
     if (!t || !h)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(t) == AGRAPH || AGTYPE(h) == AGRAPH)
+	return NULL;
+#endif
     return agfindedge(agraphof(t), t, h);
 }
 
@@ -352,6 +466,10 @@ Agnode_t *headof(Agedge_t *e)
 {
     if (!e)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH)
+	return NULL;
+#endif
     return aghead(e);
 }
 
@@ -359,6 +477,10 @@ Agnode_t *tailof(Agedge_t *e)
 {
     if (!e)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH)
+	return NULL;
+#endif
     return agtail(e);
 }
 
@@ -373,6 +495,10 @@ Agraph_t *graphof(Agedge_t *e)
 {
     if (!e)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(e) == AGRAPH)
+	return (Agraph_t*)e; /* graph of protoedge is itself recast */
+#endif
     return agraphof(agtail(e));
 }
 
@@ -380,6 +506,10 @@ Agraph_t *graphof(Agnode_t *n)
 {
     if (!n)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(n) == AGRAPH)
+	return (Agraph_t*)n;  /* graph of protonode is itself recast */
+#endif
     return agraphof(n);
 }
 
@@ -395,14 +525,22 @@ Agnode_t *protonode(Agraph_t *g)
 {
     if (!g)
         return NULL;
+#ifdef WITH_CGRAPH
+    return (Agnode_t *)g;    // gross abuse of the type system!
+#else
     return g->proto->n;
+#endif
 }
 
 Agedge_t *protoedge(Agraph_t *g)
 {
     if (!g)
         return NULL;
+#ifdef WITH_CGRAPH
+    return (Agedge_t *)g;    // gross abuse of the type system!
+#else
     return g->proto->e;
+#endif
 }
 
 //-------------------------------------------------
@@ -416,12 +554,20 @@ char *nameof(Agnode_t *n)
 {
     if (!n)
         return NULL;
+#ifdef WITH_CGRAPH
+    if (AGTYPE(n) == AGRAPH)
+	return NULL;
+#endif
     return agnameof(n);
 }
 //char *nameof(Agedge_t *e)
 //{
 //    if (!e)
 //        return NULL;
+//#ifdef WITH_CGRAPH
+//    if (AGTYPE(e) == AGRAPH)
+//	return NULL;
+//#endif
 //    return agnameof(e);
 //}
 char *nameof(Agsym_t *a)
@@ -459,12 +605,17 @@ bool ok(Agsym_t *a)
 //-------------------------------------------------
 Agraph_t *firstsubg(Agraph_t *g)
 {
+#ifndef WITH_CGRAPH
     Agraph_t *mg;
     Agnode_t *n;
     Agedge_t *e;
+#endif
 
     if (!g)
         return NULL;
+#ifdef WITH_CGRAPH
+    return agfstsubg(g);
+#else
     n = g->meta_node;
     if (!n) 
         return NULL;
@@ -475,16 +626,22 @@ Agraph_t *firstsubg(Agraph_t *g)
     if (!e) 
         return NULL;
     return agusergraph(aghead(e));
+#endif
 }
 
 Agraph_t *nextsubg(Agraph_t *g, Agraph_t *sg)
 {
+#ifndef WITH_CGRAPH
     Agraph_t *mg;
     Agnode_t *ng, *nsg;
     Agedge_t *e;
+#endif
 
     if (!g || !sg)
         return NULL;
+#ifdef WITH_CGRAPH
+    return agnxtsubg(sg);
+#else
     ng = g->meta_node;
     nsg = sg->meta_node;
     if (!ng || !nsg) 
@@ -499,8 +656,20 @@ Agraph_t *nextsubg(Agraph_t *g, Agraph_t *sg)
     if (!e) 
         return NULL;
     return agusergraph(aghead(e));
+#endif
 }
 
+#ifdef WITH_CGRAPH
+Agraph_t *firstsupg(Agraph_t *g)
+{
+    return g->parent;
+}
+
+Agraph_t *nextsupg(Agraph_t *g, Agraph_t *sg)
+{
+    return NULL;
+}
+#else
 Agraph_t *firstsupg(Agraph_t *g)
 {
     Agraph_t *mg;
@@ -544,6 +713,7 @@ Agraph_t *nextsupg(Agraph_t *g, Agraph_t *sg)
         return NULL;
     return agusergraph(agtail(e));
 }
+#endif
 
 Agedge_t *firstout(Agraph_t *g)
 {
@@ -711,7 +881,7 @@ Agnode_t *nexttail(Agnode_t *n, Agnode_t *t)
     if (!e)
         return NULL;
     do {
-        e = agnxtout(g, e);
+        e = agnxtin(g, e);
         if (!e)
             return NULL;
     } while (agtail(e) == t);
@@ -751,9 +921,13 @@ Agsym_t *firstattr(Agraph_t *g)
     if (!g)
         return NULL;
     g = agroot(g);
+#ifdef WITH_CGRAPH
+    return agnxtattr(g,AGRAPH,NULL);
+#else
     if (dtsize(g->univ->globattr->dict) == 0)
         return NULL;
     return g->univ->globattr->list[0];
+#endif
 }
 
 Agsym_t *nextattr(Agraph_t *g, Agsym_t *a)
@@ -763,6 +937,9 @@ Agsym_t *nextattr(Agraph_t *g, Agsym_t *a)
     if (!g || !a)
         return NULL;
     g = agroot(g);
+#ifdef WITH_CGRAPH
+    return agnxtattr(g,AGRAPH,a);
+#else
     for (i = 0; i < dtsize(g->univ->globattr->dict); i++)
         if (a == g->univ->globattr->list[i])
             break;
@@ -770,6 +947,7 @@ Agsym_t *nextattr(Agraph_t *g, Agsym_t *a)
     if (i > dtsize(g->univ->globattr->dict))
         return NULL;
     return g->univ->globattr->list[i];
+#endif
 }
 
 Agsym_t *firstattr(Agnode_t *n)
@@ -779,9 +957,13 @@ Agsym_t *firstattr(Agnode_t *n)
     if (!n)
         return NULL;
     g = agraphof(n);
+#ifdef WITH_CGRAPH
+    return agnxtattr(g,AGNODE,NULL);
+#else
     if (dtsize(g->univ->nodeattr->dict) == 0)
         return NULL;
     return g->univ->nodeattr->list[0];
+#endif
 }
 
 Agsym_t *nextattr(Agnode_t *n, Agsym_t *a)
@@ -792,6 +974,9 @@ Agsym_t *nextattr(Agnode_t *n, Agsym_t *a)
     if (!n || !a)
         return NULL;
     g = agraphof(n);
+#ifdef WITH_CGRAPH
+    return agnxtattr(g,AGNODE,a);
+#else
     for (i = 0; i < dtsize(g->univ->nodeattr->dict); i++)
         if (a == g->univ->nodeattr->list[i])
             break;
@@ -799,6 +984,7 @@ Agsym_t *nextattr(Agnode_t *n, Agsym_t *a)
     if (i > dtsize(g->univ->nodeattr->dict))
         return NULL;
     return g->univ->nodeattr->list[i];
+#endif
 }
 
 Agsym_t *firstattr(Agedge_t *e)
@@ -808,9 +994,13 @@ Agsym_t *firstattr(Agedge_t *e)
     if (!e)
         return NULL;
     g = agraphof(agtail(e));
+#ifdef WITH_CGRAPH
+    return agnxtattr(g,AGEDGE,NULL);
+#else
     if (dtsize(g->univ->edgeattr->dict) == 0)
         return NULL;
     return g->univ->edgeattr->list[0];
+#endif
 }
 
 Agsym_t *nextattr(Agedge_t *e, Agsym_t *a)
@@ -821,6 +1011,9 @@ Agsym_t *nextattr(Agedge_t *e, Agsym_t *a)
     if (!e || !a)
         return NULL;
     g = agraphof(agtail(e));
+#ifdef WITH_CGRAPH
+    return agnxtattr(g,AGEDGE,a);
+#else
     for (i = 0; i < dtsize(g->univ->edgeattr->dict); i++)
         if (a == g->univ->edgeattr->list[i])
             break;
@@ -828,6 +1021,7 @@ Agsym_t *nextattr(Agedge_t *e, Agsym_t *a)
     if (i > dtsize(g->univ->edgeattr->dict))
         return NULL;
     return g->univ->edgeattr->list[i];
+#endif
 }
 
 bool rm(Agraph_t *g)
@@ -836,6 +1030,16 @@ bool rm(Agraph_t *g)
 
     if (!g)
         return false;
+#ifdef WITH_CGRAPH
+    Agraph_t* sg;
+    for (sg = agfstsubg (g); sg; sg = agnxtsubg (sg))
+	rm(sg);
+    if (g == agroot(g))
+	agclose(g);
+    else
+        agdelete(agroot(g), g);
+    return true;
+#else
     if (g->meta_node) {
         for (e = agfstout(g->meta_node->graph, g->meta_node); e;
         		e = agnxtout(g->meta_node->graph, e)) {
@@ -844,12 +1048,13 @@ bool rm(Agraph_t *g)
         if (g == agroot(g)) {
             agclose(g);
         } else {
-            agdelete(g->meta_node->graph, g->meta_node);
+            agdelete(g->meta_node->graph, g);
         }
         return true;
     }
     fprintf(stderr, "subgraph has no meta_node\n");
     return false;
+#endif
 }
 
 bool rm(Agnode_t *n)
